import 'dart:async';

import 'package:fbridge/fbridge.dart';
import 'package:flite_example/mock/fbridge_mock.dart';
import 'package:flite_example/mock/router_mock.dart';
import 'package:flutter_common/flutter_common.dart';
import 'package:fflight/fflight.dart';
import 'package:flutter/material.dart';
import 'package:flutter_boost/flutter_boost.dart';
import 'package:fliggy_router/fliggy_router.dart';
import 'package:flite_tools/flitetools.dart';
import 'package:flite_tools/ui/home.dart';

void main() {
  runZoned<Future<Null>>(() async {
    runApp(MyApp());
  }, onError: (dynamic error, dynamic stackTrace) async {
    print("onError: ${error.toString()}\n${stackTrace.toString()}");
    FBridgeApi.newInstance(null).toast(error.toString(), 0);
  });
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> with FliggyModule {

  @override
  void initState() {
    super.initState();
    initApiMock();
    registerModule(this);
    initModule();
  }

  void initApiMock() {
    FliggyNavigatorApi.setProxy(FliggyNavigatorMockImp());
    FBridgeApi.setProxy((context) {
      return FBridgeMockImp(context);
    });
  }

  @override
  void onModuleRegister() {
    registerModule(FliteToolsModule());
    registerModule(FFlightModule());
  }

  @override
  void onPageRegister() {
    FliggyNavigatorApi.getInstance().registerPageBuilders({
      "/router_mock_page": (pageName, params, _) => RouterMockPage(),
    });
  }

  @override
  void onModuleInit() {
    //设置业务入口
    HomePage.setBizEnter("push /flight_dynamic_home", (context) {
      FliggyNavigatorApi.getInstance().push(context, "url", params: {
        "un_flutter": true,
        "flutter_path": "/flight_dynamic_home"
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      builder: FlutterBoost.init(),
      home: Container(color: Colors.white),
    );
  }
}
