import 'package:fbridge/fbridge.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
//import 'package:fliggy_mtop/fliggy_mtop.dart';

typedef NetCallback<T> = void Function(T data);
typedef NetErrorCallback = void Function(dynamic data);

String ERROR_MSG_NET_ERROR = "似乎未连接到互联网";
String ERROR_MSG_PARSER_ERROR = "parser error";
String ERROR_MSG_SYS_ERROR = "system error";
String ERROR_MSG_NO_RESULT = "未找到数据";
String SUCCESS = "SUCCESS";

net<T>(
  BuildContext context,
  String api,
  String version,
  Map params,
  Function paresFunction, {
  bool post = false,
  bool needLogin = false,
  VoidCallback onStart,
  NetCallback<T> onFinish,
  NetErrorCallback onFailed,
  VoidCallback onEnd,
}) async {
  Map networkRet = await FBridgeApi.newInstance(context).call("networktype");
  try {
    if (networkRet != null && networkRet['state'] != 'fail') {
      try {
        if (onStart != null) onStart();
//        MtopRequestModel request = new MtopRequestModel(api, version, params);
//        MtopResponseModel response = await FliggyMtop.getInstance().send(request);
//        if(response?.success) {
//          onFinish(paresFunction(response.data));
//        } else if(response != null) {
//          onFailed({
//            'errorMsg': response.errorCode,
//            'errorDesc': response.errorMsg,
//          });
//        } else {
//          onFailed({
//                'errorMsg': 'ERROR_MSG_SYS_ERROR',
//                'errorDesc': ERROR_MSG_SYS_ERROR,
//              });
//        }
        Map ret = await FBridgeApi.newInstance(context).call('mtop', {
          'api': api,
          'v': version,
          'post': post,
          'needLogin': needLogin,
          'data': params,
        });
        if (ret != null) {
          if (_checkError(ret)) {
            var split = ret['ret'][0].split('::');
            String errorMsg = split[0];
            String errorDesc = split[1];
            if (onFailed != null)
              onFailed({
                'errorMsg': errorMsg,
                'errorDesc': errorDesc,
              });
          } else if (ret.containsKey('result')) {
            if (onFinish != null)
              onFinish(
                  ret['result'] == null ? null : paresFunction(ret['result']));
          } else if (ret.containsKey('data')) {
            if (onFinish != null)
              onFinish(
                  ret['data'] == null ? null : (paresFunction(ret['data'])));
          } else {
            if (onFailed != null)
              onFailed({
                'errorMsg': 'ERROR_MSG_SYS_ERROR',
                'errorDesc': ERROR_MSG_SYS_ERROR,
              });
          }
        } else {
          if (onFailed != null)
            onFailed({
              'errorMsg': 'ERROR_MSG_SYS_ERROR',
              'errorDesc': ERROR_MSG_SYS_ERROR,
            });
        }
      } on PlatformException catch (e) {
        if (onFailed != null)
          onFailed({
            'errorMsg': e.details['errorMsg'],
            'errorDesc': e.details['errorDesc'],
          });
      } catch (e) {
        if (onFailed != null)
          onFailed({
            'errorMsg': 'ERROR_MSG_SYS_ERROR',
            'errorDesc': ERROR_MSG_SYS_ERROR,
          });
      }
    } else {
      if (onFailed != null)
        onFailed({
          'errorMsg': 'ERROR_MSG_NET_ERROR',
          'errorDesc': ERROR_MSG_NET_ERROR,
        });
    }
  } catch (e, stack) {
    print('error:${e.toString()}');
    print('stack:${stack.toString()}');
  }
  if (onEnd != null) onEnd();
}

bool _checkError(Map ret) {
  return ret['ret'] != null &&
      ret['ret'].length != 0 &&
      ret['ret'][0].contains("::") &&
      ret['ret'][0].split('::')[0] != SUCCESS;
}
