import 'dart:async';
import 'dart:math';
import 'package:fbridge/fbridge.dart';
import 'package:fflight/common/base_page.dart';
import 'package:fflight/common/date_utils.dart';
import 'package:fflight/common/tracker.dart';
import 'package:fflight/common/utils.dart';
import 'package:fflight/common/widget_utils.dart';
import 'package:fflight/page/flightdynamic/flight_dynamic_spm.dart';
import 'package:fflight/page/flightdynamic/net/flight_dynamic_net.dart';
import 'package:fflight/widget/dotted_line.dart';
import 'package:fflight/widget/frefresh/frefresh.dart';
import 'package:fflight/widget/fresult/fresult.dart';
import 'package:fflight/widget/fsuper/fsuper.dart';
import 'package:fflight/widget/ftip.dart';
import 'package:fflight/widget/inner_arc_clipper.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fliggy_router/fliggy_router.dart';
import 'package:ftitlebar_adapter/ftitlebar_adapter.dart';
import 'package:ftitlebar/ftitlebar.dart';

import 'net/flight_dynamic_detail_net_data.dart';
import 'net/flight_follow_data.dart';

class FlightDynamicDetailPage extends StatefulWidget {
  Map args;

  FlightDynamicDetailPage(this.args);

  @override
  _FlightDynamicDetailPageState createState() =>
      _FlightDynamicDetailPageState();
}

class _FlightDynamicDetailPageState extends State<FlightDynamicDetailPage>
    with PageTracker {
  int itemPos;
  String flightNo;
  String depDate;
  String depAirportCode;
  String arrAirportCode;

  /// 1-国内；2-国际
  int bizType;

  FlightDynamicDetailData data;

  FRefreshController refreshController;

  String refreshText = "下拉刷新";

  FTip counterInfoFloatTip;
  FTip shuttleBusFloatTip;
  GlobalKey counterInfoKey = GlobalKey();
  GlobalKey shuttleBusKey = GlobalKey();

  Widget errorView;

  bool isFirstBuild = true;
  FBridge _bridge;

  @override
  void initState() {
    super.initState();
    _bridge = FBridgeApi.newInstance(context);
    track_page(context, this);
    refreshController = FRefreshController();
    refreshController.setOnStateChangedCallback((state) {
      if (state == RefreshState.AIDL) {
        setState(() {
          refreshText = "下拉刷新";
        });
      } else if (state == RefreshState.PREPARING_REFRESH) {
        setState(() {
          refreshText = "松开刷新";
        });
      } else if (state == RefreshState.REFRESHING) {
        setState(() {
          refreshText = "正在刷新..";
        });
      }
    });
  }

  @override
  void dispose() {
    if (refreshController != null) refreshController.dispose();
    if (counterInfoFloatTip != null) counterInfoFloatTip.dispose();
    if (shuttleBusFloatTip != null) counterInfoFloatTip.dispose();
    super.dispose();
  }

  void parseArgs() {
    if (widget.args == null) return;
    itemPos = widget.args["itemPos"];
    flightNo = widget.args["flightNo"];
    depDate = widget.args["depDate"];
    depAirportCode = widget.args["depAirportCode"];
    arrAirportCode = widget.args["arrAirportCode"];
    bizType = widget.args["bizType"];
  }

  @override
  Widget build(BuildContext context) {
    if (isFirstBuild) {
      isFirstBuild = false;
      dynamic obj = ModalRoute
          .of(context)
          .settings
          .arguments;
      if (obj != null && obj is Map && obj.isNotEmpty) {
        widget.args = obj;
      }
      parseArgs();
      _requestData();
    }
    var children = <Widget>[];
    _addBackgroundPart(children);
    if (data != null) {
      var mainContentPart = _buildMainContent();
      children.add(mainContentPart);
      if (!(data.flightInfo?.attention ?? false)) {
        var bottomBtnPart = _buildBottomBtnPart();
        children.add(bottomBtnPart);
      }
    }
    return WillPopScope(
      child: Scaffold(
        appBar: FTitleBar(
          title: flightNo,
          hasMore: true,
          themeUpdateCallback: FTitleBarAdapter.themeUpdateCallback,
          moreChildren: FTitleBarAdapter.getPopupMenuItemData(),
          onMorePressed: FTitleBarAdapter.getPopupMenuPressedCallback(context),
        ),
        body: Container(
//            alignment: Alignment.center,
            color: Color(0xfff4f4f4),
            child: errorView == null
                ? Stack(
              children: children,
            )
                : Padding(
              padding: EdgeInsets.only(top: 165),
              child: errorView,
            )),
      ),
      // ignore: missing_return
      onWillPop: () {
        track_click(context, this, ctrlName: "Back");
        Map backData = {
          "itemPos": itemPos,
          "attention": data?.flightInfo?.attention ?? false,
        };
        FliggyNavigatorApi.getInstance().pop(context, result: backData);
//        Navigator.pop(context, data?.flightInfo?.attention ?? false);
        return new Future.value(false);
      },
    );
  }

  void _addBackgroundPart(List<Widget> children) {
    children.add(Container(
      height: 250,
      decoration: BoxDecoration(
          gradient: LinearGradient(
            colors: [
              Color(0xffffd800),
              Color(0xffffc900),
            ],
            begin: Alignment.topCenter,
            end: Alignment.bottomCenter,
          )),
    ));
    children.add(Container(
      height: 250,
      decoration: BoxDecoration(
          gradient: LinearGradient(
            colors: [
              Color(0x00f4f4f4),
              Color(0xfff4f4f4),
            ],
            begin: Alignment.topCenter,
            end: Alignment.bottomCenter,
          )),
    ));
  }

  Widget _buildMainContent() {
    var children = <Widget>[];
    children.add(_buildPart1());
    children.add(_buildPart2());

    /// 温馨提示
    if (textNotEmpty(data.flightInfo?.showData?.jourWarnTips)) {
      children.add(Padding(
        padding:
        const EdgeInsets.only(left: 18, right: 18, top: 15, bottom: 12),
        child: Text(
          '${safeString(data.flightInfo?.showData?.jourWarnTips)}',
          style: TextStyle(
            color: Color(0xff999999),
            fontSize: 12,
          ),
        ),
      ));
    }

    /// 取消关注
    if ((data.flightInfo?.attention) ?? false) {
      children.add(FSuper(
        width: double.infinity,
        height: 48,
        margin: EdgeInsets.only(left: 6, right: 6),
        text: '取消关注',
        textColor: Color(0xff999999),
        textSize: 18,
        textAlignment: Alignment.center,
        strokeColor: Color(0xffe0e0e0),
        strokeWidth: 1,
        corner: Corner.all(3),
        backgroundColor: Colors.white,
        onClick: _requestFollow,
      ));
    }
    return FRefresh(
      controller: refreshController,
      header: _buildRefreshHeader(),
      headerHeight: 119,
      triggerOffset: 119,
      child: Column(
        children: children,
      ),
      onRefresh: () {
        _requestData();
      },
    );
  }

  Widget _buildRefreshHeader() {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Image.asset("images/pull_refresh.gif", package: "fflight", width: 100, height: 100,),
        Text(
          refreshText,
          style: TextStyle(fontSize: 11, color: Color(0xff999999)),
        ),
      ],
    );
  }

  Widget _buildPart1() {
    return Padding(
//      height: 324,
      padding: EdgeInsets.only(left: 6, right: 6, top: 6, bottom: 12),
      child: Column(
        children: <Widget>[

          /// 上半部分
          Container(
            padding:
            EdgeInsets.only(top: 15.0 - textSpace(13), left: 15, right: 15),
            decoration: BoxDecoration(
              color: Color(0xffffffff),
              borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(9), topRight: Radius.circular(9)),
            ),
            child: Column(
              children: <Widget>[

                /// 航司和日期
                Padding(
                  padding: EdgeInsets.only(bottom: 12.0 - textSpace(13)),
                  child: _buildFlightNumberPart(),
                ),

                /// 状态信息
                _buildFlightStatePart(),

                /// 城市
                Padding(
                  padding: safeEdgeInsets(bottom: 6.0 - textSpace(12.0 + 15.0)),
                  child: _buildCityPart(),
                ),

                /// 机场
                Padding(
                  padding:
                  EdgeInsets.only(bottom: 22.0 - textSpace(15.0 + 12.0)),
                  child: _buildAirportPart(),
                ),

                /// 时间状态
                Padding(
                  padding: safeEdgeInsets(bottom: 3.0 - textSpace(12)),
                  child: _buildTimeStatePart(),
                ),

                /// 时间
                Padding(
                  padding: safeEdgeInsets(bottom: 6.0 - textSpace(12)),
                  child: _buildTimePart(),
                ),

                /// 时间补充信息
                Padding(
                  padding: EdgeInsets.only(bottom: 24.0 - textSpace(12)),
                  child: _buildTimeDetailPart(),
                ),
              ],
            ),
          ),

          /// 虚线部分
          _buildDottedLinePart(),
          Container(
            padding: EdgeInsets.only(
                top: 7 - textSpace(12), left: 15, right: 15, bottom: 12),
            decoration: BoxDecoration(
              color: Color(0xffffffff),
              borderRadius: BorderRadius.only(
                  bottomLeft: Radius.circular(9),
                  bottomRight: Radius.circular(9)),
            ),
            child: _buildMoreInfoPart(),
          )
        ],
      ),
    );
  }

  Widget _buildFlightNumberPart() {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: <Widget>[
        FSuper(
          text:
          '${safeString(data.flightInfo?.airlineName)} ${safeString(
              data.flightInfo?.flightNo)}',
          textColor: Color(0xff666666),
          textSize: 13,
          child1: Image.network(
            data.flightInfo?.airlineIcon,
            width: 15,
            height: 15,
          ),
          child1Alignment: Alignment.centerLeft,
          padding: EdgeInsets.only(left: 18),
        ),
        Text(
          '${stringDateFormat(
              data.flightInfo?.depTime, 'MM月dd日', isTwin: false)} ${weekdayCN(
              getWeekday(data.flightInfo?.depTime))}',
          style: TextStyle(
            color: Color(0xff999999),
            fontSize: 13,
          ),
        ),
      ],
    );
  }

  Widget _buildFlightStatePart() {
    return Container(
      height: 60,
      padding: EdgeInsets.only(left: 18, right: 9),
      margin: EdgeInsets.only(bottom: 8),
      decoration: BoxDecoration(
        color: color(data.flightInfo?.showData?.flightStatusColor,
            defaultColor: '#2ea3ff'),
        borderRadius: BorderRadius.all(Radius.circular(3)),
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        children: <Widget>[
          Expanded(
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                Text(
                  safeString(data.flightInfo?.flightStatus),
                  style: TextStyle(
                    color: Colors.white,
                    fontSize: 25,
                  ),
                ),
                const SizedBox(width: 18),
                verticalDivider(height: 24, width: 0.5, color: Colors.white),
                const SizedBox(width: 12),
                Text(
                  safeString(data.flightInfo?.showData?.flightStatusTips),
                  textAlign: TextAlign.left,
                  style: TextStyle(
                    color: Colors.white,
                    fontSize: 11,
                  ),
                )
              ],
            ),
          ),
          FSuper(
            text: '航班\n跟踪',
            textColor: Color(0xb3ffffff),
            textSize: 11,
            padding: EdgeInsets.only(right: 16),
            margin: EdgeInsets.only(left: 20),
            child1: Icon(
              Icons.arrow_forward_ios,
              size: 14,
              color: Colors.white,
            ),
            child1Alignment: Alignment.centerRight,
            onClick: _handleFlightTrackBtnClicked,
          )
        ],
      ),
    );
  }

  Widget _buildCityPart() {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: <Widget>[
        Text(
          safeString(data.flightInfo?.depCity),
          style: TextStyle(
            color: Color(0xff999999),
            fontSize: 12,
          ),
        ),
        Text(
          safeString(data.flightInfo?.arrCity),
          style: TextStyle(
            color: Color(0xff999999),
            fontSize: 12,
          ),
        ),
      ],
    );
  }

  Widget _buildAirportPart() {
    var children = <Widget>[];
    children.add(Align(
      alignment: Alignment.centerLeft,
      child: Text(
        '${safeString(data.flightInfo?.depAirport)}${safeString(
            data.flightInfo?.depTerm)}',
        style: TextStyle(
          color: Color(0xff666666),
          fontSize: 15,
        ),
      ),
    ));
    if (textNotEmpty(data.flightInfo?.showData?.landTips) &&
        data.flightInfo?.showData?.landTips != '--') {
      children.add(Text(
        safeString(data.flightInfo?.showData?.landTips),
        style: TextStyle(
          color: Color(0xfffca500),
          fontSize: 15,
        ),
      ));
    }
    children.add(Align(
      alignment: Alignment.centerRight,
      child: Text(
        '${safeString(data.flightInfo?.arrAirport)}${safeString(
            data.flightInfo?.arrTerm)}',
        style: TextStyle(
          color: Color(0xff666666),
          fontSize: 15,
        ),
      ),
    ));
    return Stack(
      alignment: Alignment.center,
      children: children,
    );
  }

  Widget _buildTimeStatePart() {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: <Widget>[
        Text(
          safeString(data.scheduleInfo?.firstRawDep),
          style: TextStyle(
            color: Color(0xff999999),
            fontSize: 12,
          ),
        ),
        Text(
          safeString(data.scheduleInfo?.firstRawArr),
          style: TextStyle(
            color: Color(0xff999999),
            fontSize: 12,
          ),
        ),
      ],
    );
  }

  Widget _buildTimePart() {
    var children = <Widget>[];
    var depTime = data.scheduleInfo?.firstRawDepTime ?? '';
    depTime = stringDateFormat(depTime, 'hh:mm');
    children.add(Align(
      alignment: Alignment.centerLeft,
      child: Text(
        depTime,
        style: TextStyle(
          color: Color(0xff3d3d3d),
          fontSize: 30,
          height: 1,
        ),
      ),
    ));
    children.add(images(
      'icon_flight_1.png',
      width: 24,
      height: 24,
      color: Color(0xffcccccc),
      colorBlendMode: BlendMode.srcIn,
    ));
    var arrTime = data.scheduleInfo?.firstRawArrTime ?? '';
    arrTime = stringDateFormat(arrTime, 'hh:mm');
    int dateInterval = stringDaysBetween(
        data.scheduleInfo?.arrTime, data.scheduleInfo?.depTime);
    if (dateInterval > 0) {
      var arrDate =
          '${stringDateFormat(
          data.scheduleInfo?.arrTime, 'MM-dd')}\n${weekdayCN(
          getWeekday(data.scheduleInfo?.arrTime))}';
      children.add(Align(
        alignment: Alignment.centerRight,
        child: Row(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              arrDate,
              textAlign: TextAlign.right,
              style: TextStyle(
                color: Color(0xff3d3d3d),
                fontSize: 10,
                height: 1,
              ),
            ),
            const SizedBox(width: 3),
            Text(
              arrTime,
              style: TextStyle(
                color: Color(0xff3d3d3d),
                fontSize: 30,
                height: 1,
              ),
            ),
          ],
        ),
      ));
    } else {
      children.add(Align(
        alignment: Alignment.centerRight,
        child: Text(
          arrTime,
          style: TextStyle(
            color: Color(0xff3d3d3d),
            fontSize: 30,
            height: 1,
          ),
        ),
      ));
    }
    return Stack(
      alignment: Alignment.center,
      children: children,
    );
  }

  Widget _buildTimeDetailPart() {
    var children = <Widget>[];
    String dep =
        '${safeString(data.scheduleInfo?.secondRawDep)} ${stringDateFormat(
        data.scheduleInfo?.secondRawDepTime ?? '', 'hh:mm')}';
    children.add(Align(
      alignment: Alignment.centerLeft,
      child: Text(
        dep,
        style: TextStyle(
          color: Color(0xff999999),
          fontSize: 12,
        ),
      ),
    ));
    if (textNotEmpty(data.scheduleInfo?.consumeTime ?? '')) {
      children.add(Text(
        data.scheduleInfo?.consumeTime,
        style: TextStyle(
          color: Color(0xff999999),
          fontSize: 12,
        ),
      ));
    }
    int dateInterval = stringDaysBetween(
        data.scheduleInfo?.arrTime, data.scheduleInfo?.depTime);
    var arrDate;
    if (dateInterval > 0) {
      arrDate = stringDateFormat(data.scheduleInfo?.arrPlanTime, 'MM月dd日',
          isTwin: false);
      arrDate = ' $arrDate';
    }
    String arr =
        '${safeString(data.scheduleInfo?.secondRawArr)}${arrDate ??
        ''} ${stringDateFormat(
        data.scheduleInfo?.secondRawArrTime ?? '', 'hh:mm')}';
    children.add(Align(
      alignment: Alignment.centerRight,
      child: Text(
        arr,
        style: TextStyle(
          color: Color(0xff999999),
          fontSize: 12,
        ),
      ),
    ));
    return Stack(
      alignment: Alignment.center,
      children: children,
    );
  }

  Widget _buildDottedLinePart() {
    return ClipPath(
      clipper: InnerArcClipper(),
      child: Container(
        color: Colors.white,
        height: 16,
        padding: EdgeInsets.only(left: 15, right: 15),
        alignment: Alignment.center,
        child: DottedLine(
          color: Color(0xfff1f1f1),
          height: 1,
        ),
      ),
    );
  }

  Widget _buildMoreInfoPart() {
    var counterInfo =
    safeString(data.checkInfo?.counterInfo, defaultString: "--");
    var gate = safeString(data.checkInfo?.gate, defaultString: '--');
    var carousel = safeString(data.checkInfo?.carousel, defaultString: "--");
    final counterInfoSpan = TextSpan(
        text: counterInfo,
        style: TextStyle(
          color: getTextColor(counterInfo),
          fontSize: 15,
          height: 1,
        ));
    final tp = TextPainter(
        text: counterInfoSpan, maxLines: 1, textDirection: TextDirection.ltr);
    tp.layout(maxWidth: 105.5);
    Widget counterInfoText = Text(
      counterInfo,
      key: counterInfoKey,
      style: TextStyle(
        color: getTextColor(counterInfo),
        fontSize: 15,
        height: 1,
        decoration: tp.didExceedMaxLines ? TextDecoration.underline : null,
      ),
      maxLines: 1,
      overflow: TextOverflow.ellipsis,
    );
    if (tp.didExceedMaxLines) {
      counterInfoText = InkWell(
        onTap: () {
          _showCounterInfoDetail(context, counterInfo);
        },
        child: counterInfoText,
      );
    }
    var w = (MediaQuery
        .of(context)
        .size
        .width - 30.0 - 12.0 - 1.0) / 3.0;
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: <Widget>[
        Container(
          width: w,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Text(
                '值机柜台',
                style: TextStyle(
                  color: Color(0xff999999),
                  fontSize: 12,
                  height: 1,
                ),
              ),
              SizedBox(height: 6),
              counterInfoText,
            ],
          ),
        ),
        verticalDivider(height: 35.5, width: 0.5, color: Color(0xfff4f4f4)),
        Container(
          width: w,
          padding: EdgeInsets.only(left: 12),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Text(
                '登机口',
                style: TextStyle(
                  color: Color(0xff999999),
                  fontSize: 12,
                  height: 1,
                ),
              ),
              SizedBox(height: 4.5),
              IntrinsicWidth(
                child: FSuper(
                  height: 18,
                  text: gate,
                  textColor: getTextColor(gate),
                  textSize: 15,
                  textAlignment: Alignment.centerLeft,
                  child1: (data.checkInfo?.isShuttleBus ?? false)
                      ? images(
                    'icon_qiche.png',
                    key: shuttleBusKey,
                    width: 18,
                    height: 18,
                  )
                      : null,
                  padding: EdgeInsets.only(right: 21),
                  child1Alignment: Alignment.centerRight,
                  onChild1Click: () {
                    _showShuttleBusTips(context);
                  },
                ),
              )
            ],
          ),
        ),
        verticalDivider(height: 30, width: 0.5, color: Color(0xfff4f4f4)),
        Container(
          width: w,
          padding: EdgeInsets.only(left: 12),
          child: _buildMoreInfoItem('行李转盘', carousel),
        ),
      ],
    );
  }

  Widget _buildMoreInfoItem(String title, String value) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Text(
          title,
          style: TextStyle(
            color: Color(0xff999999),
            fontSize: 12,
            height: 1,
          ),
        ),
        SizedBox(height: 6),
        Text(
          value,
          style: TextStyle(
            color: getTextColor(value),
            fontSize: 15,
            height: 1,
          ),
          maxLines: 1,
          overflow: TextOverflow.ellipsis,
        ),
      ],
    );
  }

  Widget _buildPart2() {
    var planeType = safeString(data.planeInfo?.planeType, defaultString: '--');
    var onTimeRateHis =
    safeString(data.flightInfo?.onTimeRateHis, defaultString: '--');
    var meal = safeString(data.flightInfo?.meal, defaultString: '--');
    var w = (MediaQuery
        .of(context)
        .size
        .width - 30.0 - 12.0 - 1.0) / 3.0;
    return Container(
      height: 61,
      alignment: Alignment.center,
      margin: EdgeInsets.only(left: 6, right: 6),
      decoration: BoxDecoration(
        color: Color(0xffffffff),
        borderRadius: BorderRadius.all(Radius.circular(3)),
      ),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          FSuper(
            width: w,
            padding: EdgeInsets.only(left: 15),
            text: '机型',
            textColor: Color(0xff999999),
            textSize: 12,
            textAlign: TextAlign.left,
            spans: [
              FSuper.VerticalSpace(1),
              TextSpan(
                text: planeType,
                style: TextStyle(
                  color: getTextColor(planeType),
                  fontSize: 13,
                ),
              )
            ],
          ),
          verticalDivider(height: 30, width: 0.5, color: Color(0xfff4f4f4)),
          FSuper(
            width: w,
            padding: EdgeInsets.only(left: 12),
            text: '餐食',
            textColor: Color(0xff999999),
            textSize: 12,
            textAlign: TextAlign.left,
            spans: [
              FSuper.VerticalSpace(1),
              TextSpan(
                text: meal,
                style: TextStyle(
                  color: getTextColor(meal),
                  fontSize: 13,
                ),
              )
            ],
          ),
          verticalDivider(height: 30, width: 0.5, color: Color(0xfff4f4f4)),
          FSuper(
            width: w,
            padding: EdgeInsets.only(left: 12),
            text: '历史准点率',
            textColor: Color(0xff999999),
            textSize: 12,
            textAlign: TextAlign.left,
            spans: [
              FSuper.VerticalSpace(1),
              TextSpan(
                text: onTimeRateHis,
                style: TextStyle(
                  color: getTextColor(onTimeRateHis),
                  fontSize: 13,
                ),
              )
            ],
          ),
        ],
      ),
    );
  }

  Color getTextColor(String text) {
    switch (text) {
      case '--':
      case '待定':
      case '未知':
        return Color(0xffcccccc);
    }
    return Color(0xff666666);
  }

  Widget _buildBottomBtnPart() {
    return SafeArea(
        child: Align(
          alignment: Alignment.bottomCenter,
          child: FSuper(
            width: double.infinity,
            height: 48,
            textColor: Colors.white,
            textSize: 18,
            text: '添加关注',
            textAlignment: Alignment.center,
            gradient: LinearGradient(colors: [
              Color(0xffff9300),
              Color(0xffff3355),
            ]),
            onClick: _requestFollow,
          ),
        )
    );
  }

  _requestData() {
    showFLoading(context);
    FlightDynamicNet.requestDynamicDetail<FlightDynamicDetailNetData>(
      flightNo: flightNo,
      flightDate: depDate,
      depAirportCode: depAirportCode,
      arrAirportCode: arrAirportCode,
      bizType: bizType?.toString(),
      onFinish: (data) {
        if (data != null && data.result != null) {
          errorView = null;
          setState(() {
            this.data = data.result;
            flightNo = this.data.flightInfo?.flightNo;
            bizType = (this.data.flightInfo?.national ?? false) ? 1 : 2;
          });
        } else {
          this.data = null;
          setState(() {
            errorView = FResult.noGoods(
              'widget/fresult/',
              'fflight',
              onButtonPress: () {},
              errorText: "亲，当前无航班数据",
//              showViceError: false,
//              showBtnContainer: false,
            );
          });
        }
      },
      onFailed: (error) {
        switch (error['errorMsg']) {
          case 'ERROR_MSG_NET_ERROR':
            setState(() {
              errorView = FResult.noNetwork('widget/fresult/', 'fflight',
                  onButtonPress: () {
                    _requestData();
                  }, onViceBtnPress: null);
            });
            break;
          default:
            setState(() {
              errorView = FResult.noGoods(
                'widget/fresult/',
                'fflight',
                onButtonPress: () {},
                errorText: error['errorDesc'],
              );
            });
        }
      },
      onEnd: () {
        hideFLoading(context);
        if (refreshController != null && refreshController.refreshing) {
          refreshController.finish();
        }
      },
    );
  }

  void _requestFollow() async {
    showFLoading(context);
    var attendType = (data?.flightInfo?.attention ?? false) ? '-1' : '1';
    if (attendType == "1") {
      track_click(context, this,
          ctrlName: FlightDynamicSpm.Add_attention_detail.name,
          ctrlSpm: FlightDynamicSpm.Add_attention_detail.spm);
    } else {
      track_click(context, this,
          ctrlName: FlightDynamicSpm.Cancelled_attention_detail.name,
          ctrlSpm: FlightDynamicSpm.Cancelled_attention_detail.spm);
    }
    FlightDynamicNet.requestAttention<FlightFollowData>(
      flightNo: data?.flightInfo?.flightNo,
      flightDate: data?.flightInfo?.depTime,
      depAirportCode: data?.flightInfo?.depAirportCode,
      arrAirportCode: data?.flightInfo?.arrAirportCode,
      attendType: attendType,
      onFinish: (data) {
        if (data != null && !data.failure) {
          setState(() {
            this.data?.flightInfo?.attention = attendType == '1';
            if ((this.data?.flightInfo?.attention ?? false)) {
              _bridge.toast('关注成功');
            } else {
              _bridge.toast('取消关注成功');
            }
          });
        }
      },
      onFailed: (error) {
        if(error is Map && error['errorDesc'] != "system error"){
          _bridge.toast(error['errorDesc']);
        }
      },
      onEnd: () => hideFLoading(context),
    );
  }

  void _handleFlightTrackBtnClicked() {
    String flightNo = data?.flightInfo?.flightNo;
    String flightDate = data?.flightInfo?.depTime;
    String depAirportCode = data?.flightInfo?.depAirportCode;
    String arrAirportCode = data?.flightInfo?.arrAirportCode;
    String orderId = data?.orderId;
    int bizType = this.bizType;
    Map args = {
      'flightNo': flightNo,
      'depDate': flightDate,
      'depAirportCode': depAirportCode,
      'arrAirportCode': arrAirportCode,
      'bizType': bizType,
      'orderId': orderId
    };
    openFlutterPage(context, "flight_dynamic_info_trajectory", args);
//    Navigator.push(context, MaterialPageRoute(
//      builder: (context) {
//
//        return FlightDynamicLocusPage(args);
//      },
//    ));
  }

  void _showCounterInfoDetail(BuildContext context, String text) {
    RenderBox box = counterInfoKey.currentContext?.findRenderObject();
    Offset offset = box?.localToGlobal(Offset.zero);
    if (counterInfoFloatTip == null) {
      counterInfoFloatTip = FTip(
        text: text,
        width: 145,
        leftOffset: 20.5,
        topOffset: (offset?.dy ?? 0) + (box?.size?.height ?? 0) + 7.5,
        canClickDismiss: true,
        autoDismiss: true,
        triangleOffset: 42,
      ).show(context);
    } else {
      counterInfoFloatTip..text = text;
      counterInfoFloatTip.show(context);
    }
  }

  void _showShuttleBusTips(BuildContext context) {
    String text = "需乘摆渡车";
    RenderBox box = shuttleBusKey.currentContext?.findRenderObject();
    Offset offset = box?.localToGlobal(Offset.zero);
    print('offset?.dx = ${offset.dx}');
    print('width = ${box.size}');
    if (shuttleBusFloatTip == null) {
      shuttleBusFloatTip = FTip(
        text: text,
        width: 95.5,
        leftOffset: (offset?.dx ?? 0) + 9 - 95.5 / 2.0,
        topOffset: (offset?.dy ?? 0) + (box?.size?.height ?? 0) + 7.5,
        canClickDismiss: true,
        autoDismiss: true,
      ).show(context);
    } else {
      shuttleBusFloatTip.show(context);
    }
  }

  @override
  String getPageName() {
    return "flight_dynamic_detail";
  }

  @override
  String getPageSpmCnt() {
    return "181.11332749.0.0";
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
      track_page(context, this);
    }
  }
}
