import 'dart:convert';
import 'dart:math';
import 'dart:io';
import 'package:fbridge/fbridge.dart';
import 'package:fflight/common/base_page.dart';
import 'package:fflight/common/date_utils.dart';
import 'package:fflight/common/reg_utils.dart';
import 'package:fflight/common/tracker.dart';
import 'package:fflight/common/utils.dart';
import 'package:fflight/common/widget_utils.dart';
import 'package:fflight/page/flightdynamic/net/flight_dynamic_list_net_data.dart';
import 'package:fflight/widget/flightdynamiclistcell/flight_dynamic_list_cell_data.dart';
import 'package:fflight/widget/flightdynamiclistcell/flight_dynamic_list_cell_view.dart';
import 'package:fflight/widget/fsuper/fsuper.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:fliggy_router/fliggy_router.dart';
import 'package:ftitlebar_adapter/ftitlebar_adapter.dart';
import 'package:ftitlebar/ftitlebar.dart';
import 'package:flutter_boost/flutter_boost.dart';

import 'flight_dynamic_spm.dart';
import 'net/flight_dynamic_net.dart';

class FlightDynamicHomePage extends StatefulWidget {
  @override
  _FlightDynamicHomePageState createState() => _FlightDynamicHomePageState();
}

class _FlightDynamicHomePageState extends State<FlightDynamicHomePage>
    with SingleTickerProviderStateMixin, PageTracker {
  static const String CODE_LOGIN = 'CODE_LOGIN';
  TabController _controller;
  TextEditingController _inputFlightNoController;
  FocusNode _commentFocus = FocusNode();

  String flightNo = '';
  String flightSearchDate = dateFormat(DateTime.now(), 'yyyy-MM-dd');
  String locationSearchDate = dateFormat(DateTime.now(), 'yyyy-MM-dd');
  String depCityName = "北京";
  String depCityCode = "BJS";
  String arrCityName = "杭州";
  String arrCityCode = "HGH";
  Map userInfo;
  List<FlightDynamicListCellData> datas;

  Alignment cityAlignment_1 = Alignment.centerLeft;
  Alignment cityAlignment_2 = Alignment.centerRight;
  bool cityExchanging = false;
  bool isFirstBuild = true;

  FBridge _bridge;

  void boostContainerLifeCycleObserver(ContainerLifeCycle state, BoostContainerSettings settings) {
    if (settings.name.endsWith(getPageName())) {
      debugPrint("boostContainerLifeCycleObserver: ${state.toString()}, ${settings.name}");
      if (state == ContainerLifeCycle.Appear) {
        _loginCallback();
      } else if (state == ContainerLifeCycle.Disappear) {
        FocusScope.of(context).requestFocus(FocusNode());
      }
    }
  }

  @override
  void initState() {
    super.initState();
    _bridge = FBridgeApi.newInstance(context);
    track_page(context, this);
    FlutterBoost.singleton.addBoostContainerLifeCycleObserver(boostContainerLifeCycleObserver);

    _controller = TabController(initialIndex: 0, length: 2, vsync: this);
    _controller.addListener(_onTabChangedListener);
    _inputFlightNoController = TextEditingController();
    _getUser();
  }

  @override
  void deactivate() {
    super.deactivate();
    FlutterBoost.singleton.observersHolder.removeObserver(boostContainerLifeCycleObserver);
  }

  @override
  void dispose() {
    _controller.dispose();
    _inputFlightNoController.dispose();
    super.dispose();
  }

  _getUser() async {
    userInfo = await _bridge.call("user_info");
  }

  _onTabChangedListener() {
    if (_controller.indexIsChanging) {
      if (mounted) {
        setState(() {});
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: FTitleBar(
          title: '航班动态',
          hasMore: true,
          themeUpdateCallback: FTitleBarAdapter.themeUpdateCallback,
          moreChildren: FTitleBarAdapter.getPopupMenuItemData(),
          onMorePressed: FTitleBarAdapter.getPopupMenuPressedCallback(context),
        ),
        body: Container(
          height: double.infinity,
          color: Color(0xfff4f4f4),
          child: SingleChildScrollView(
            physics: BouncingScrollPhysics(),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                _buildHeaderPart(),
//                Visibility(
//                  visible: hasShow(),
//                  child: Padding(
//                    padding:
//                        const EdgeInsets.only(left: 12, top: 27, bottom: 6),
//                    child: Text(
//                      '我关注的航班',
//                      style: TextStyle(
//                        color: Color(0xff999999),
//                        fontSize: 13,
//                      ),
//                    ),
//                  ),
//                ),
                _buildAttentionPart(),
              ],
            ),
          ),
        ));
  }

  bool hasShow() {
    if (hasLogin()) {
      if (datas != null && datas.length > 0) {
        return true;
      } else {
        return false;
      }
    } else {
      return true;
    }
  }

  Widget _buildAttentionPart() {
    return StatefulBuilder(builder: (context, state) {
      if (isFirstBuild) {
        isFirstBuild = false;
        firstBuildAttentionPart(state);
        return EMPTY_WIDGET;
      }
      if (hasLogin()) {
        if (datas != null && datas.length > 0) {
          return Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Padding(
                padding: const EdgeInsets.only(left: 12, top: 27, bottom: 6),
                child: Text(
                  '我关注的航班',
                  style: TextStyle(
                    color: Color(0xff999999),
                    fontSize: 13,
                  ),
                ),
              ),
              _buildAttentionList(),
            ],
          );
        } else {
          return EMPTY_WIDGET;
        }
      } else {
        return Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Padding(
              padding: const EdgeInsets.only(left: 12, top: 27, bottom: 6),
              child: Text(
                '我关注的航班',
                style: TextStyle(
                  color: Color(0xff999999),
                  fontSize: 13,
                ),
              ),
            ),
            _buildLogin(),
          ],
        );
      }
    });
  }

  firstBuildAttentionPart(StateSetter setter) async {
    await checkUser(setter);
    if (!hasLogin()) {
      setter(() {});
    }
  }

  checkUser(StateSetter state) async {
    if (!hasLogin()) {
      await _getUser();
    }
    requestData(state);
  }

  bool hasLogin() {
    return userInfo != null &&
        textNotEmpty(userInfo['userId']) &&
        textNotEmpty(userInfo['userName']);
  }

  Widget _buildAttentionList() {
    List<Widget> children = [];
    for (int i = 0; i < datas.length; i++) {
      children.add(_buildListCell(context, i));
    }
    return Column(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: children,
    );
//    return Expanded(
//      child: ListView.builder(
//        itemCount: datas.length,
//        itemBuilder: _buildListCell,
//        physics: BouncingScrollPhysics(),
//      ),
//    );
  }

  Widget _buildListCell(BuildContext context, int index) {
    var cell = StatefulBuilder(builder: (context, state) {
      return InkWell(
        onTap: () async {
          FlightDynamicListCellData data = datas[index];
          Map args = {
            'itemPos': index,
            'flightNo': data.flightNo,
            'depDate': data.depTime,
            'depAirportCode': data.depAirportCode,
            'arrAirportCode': data.arrAirportCode,
            'bizType': data.national ? 1 : 2,
          };
          Map result = await openFlutterPage(context, "flight_dynamic_detail", args);
          _loginCallback();
//          Navigator.push(context, MaterialPageRoute(builder: (context) {
//            FlightDynamicListCellData data = datas[index];
//
//            return FlightDynamicDetailPage(args);
//          }));
        },
        child: Padding(
          padding: const EdgeInsets.only(bottom: 6),
          child: FlightDynamicListCellView(
            datas[index],
            showAttention: false,
            showCity: true,
            showDate: true,
          ),
        ),
      );
    });
    return cell;
  }

  Widget _buildLogin() {
    return Container(
      margin: EdgeInsets.only(top: 20.0, bottom: 10.0),
      padding: EdgeInsets.only(top: 25, bottom: 20),
      width: double.infinity,
      color: Colors.white,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Text(
            "请登录查看已关注的航班动态哦",
            style: TextStyle(color: Color(0xff999999), fontSize: 13),
          ),
          InkWell(
            onTap: () {
              FliggyNavigatorApi.getInstance().push(context, "login", params:{'animeType': 1});
            },
            child: Padding(
              padding: const EdgeInsets.only(top: 6, bottom: 6),
              child: FSuper(
                text: '登录',
                textColor: Color(0xffee9900),
                padding:
                EdgeInsets.only(top: 3, bottom: 3, left: 15, right: 15),
                margin: EdgeInsets.only(top: 15),
                corner: Corner.all(3),
                strokeColor: Color(0xffee9900),
                textSize: 16,
                strokeWidth: 1,
              ),
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildHeaderPart() {
    var inputConditionWidget = _controller.index == 0
        ? _buildFlightSearchView()
        : _buildLocationSearchView();
    return Container(
      color: Colors.white,
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[

          /// TabBar
          _buildTabBar(),
          Divider(
            height: 0.5,
            color: Color(0xffe0e0e0),
          ),
          inputConditionWidget,

          /// 搜索按钮
          FSuper(
            width: double.infinity,
            height: 44,
            margin: EdgeInsets.only(top: 16, bottom: 17, left: 12, right: 12),
            text: '查询',
            textSize: 18,
            textColor: Color(0xffffffff),
            textAlignment: Alignment.center,
            gradient: LinearGradient(colors: [
              Color(0xffff9300),
              Color(0xffff3355),
            ]),
            corner: Corner.all(3),
            onClick: _onSearchButtonClick,
          ),
        ],
      ),
    );
  }

  Widget _buildTabBar() {
    return TabBar(
      controller: _controller,
      tabs: [
        Tab(
          text: "按航班号",
        ),
        Tab(
          text: "按起降地",
        ),
      ],
      labelColor: Color(0xff333333),
      labelStyle: TextStyle(
        fontSize: 14,
        fontWeight: FontWeight.bold,
      ),
      unselectedLabelColor: Color(0xff333333),
      unselectedLabelStyle: TextStyle(
        fontSize: 14,
        fontWeight: FontWeight.normal,
      ),
      indicatorWeight: 3,
      indicatorSize: TabBarIndicatorSize.label,
      indicatorColor: Color(0xffffc900),
    );
  }

  Widget _buildFlightSearchView() {
    cityExchanging = false;
    return Padding(
      padding: EdgeInsets.only(left: 12, right: 12),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[

          /// 航班号输入框
          Container(
            height: 51,
            child: Stack(
              alignment: Alignment.centerRight,
              children: <Widget>[
                TextField(
                  focusNode: _commentFocus,
                  keyboardType: TextInputType.text,
                  controller: _inputFlightNoController,
                  textCapitalization: TextCapitalization.characters,
                  style: TextStyle(
                    color: Color(0xff333333),
                    fontSize: 21,
                  ),
                  decoration: InputDecoration(
                    hintText: "请输入航班号，如CA1710",
                    hintStyle: TextStyle(
                      fontSize: 21,
                      color: Color(0xffcccccc),
                    ),
                    border: OutlineInputBorder(
                      borderSide: BorderSide.none,
                    ),
                    contentPadding: EdgeInsets.all(0),
                  ),
                  inputFormatters: [
                    WhitelistingTextInputFormatterWithUppercase(
                        RegExp(REG_ALPHABET_NUMBER)),
                  ],
                  onChanged: (text) {
                    flightNo = text;
                  },
                ),
                InkWell(
                  child: Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Image.asset(
                      "images/icon_close.png",
                      package: "fflight",
                      width: 12,
                      height: 12,
                    ),
                  ),
                  onTap: _handleFlightNumberClearClicked,
                ),
              ],
            ),
          ),
          Divider(
            height: 0.5,
            color: Color(0xffe0e0e0),
          ),

          /// 日期
          FSuper(
            width: double.infinity,
            height: 51,
            text: stringDateFormat(flightSearchDate, 'MM月dd日', isTwin: false),
            textAlignment: Alignment.centerLeft,
            textColor: Color(0xff202325),
            textSize: 21,
            spans: [
              FSuper.HorizontalSpace(6),
              TextSpan(
                  text: weekdayCN(getWeekday(flightSearchDate)),
                  style: TextStyle(
                    color: Color(0xff999999),
                    fontSize: 13,
                  ))
            ],
            child1: Icon(
              Icons.arrow_forward_ios,
              size: 12,
              color: Color(0xff999999),
            ),
            child1Alignment: Alignment.centerRight,
            onClick: _handlerDateClicked,
          ),
          Divider(
            height: 0.5,
            color: Color(0xffe0e0e0),
          ),
        ],
      ),
    );
  }

  Widget _buildLocationSearchView() {
    var clickAreaW =
        (MediaQuery
            .of(context)
            .size
            .width - 35.7 - 24.0 - 4) / 2.0;
    return Padding(
      padding: EdgeInsets.only(left: 12, right: 12),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[

          /// 出发到达选择
          StatefulBuilder(
            builder: (context, state) {
              String cityName_1 = safeString(
                  cityAlignment_1 == Alignment.centerLeft
                      ? depCityName
                      : arrCityName,
                  defaultString: '北京');
              String cityName_2 = safeString(
                  cityAlignment_2 == Alignment.centerLeft
                      ? depCityName
                      : arrCityName,
                  defaultString: '杭州');
              return Stack(
                children: <Widget>[
                  Container(
                    height: 51,
                    child: AnimatedContainer(
                      duration: Duration(milliseconds: 300),
                      curve: Curves.decelerate,
                      alignment: cityAlignment_1,
                      onEnd: () {
                        cityExchanging = false;
                      },
                      child: Text(
                        cityName_1,
                        style: TextStyle(
                            color: Color(0xff333333),
                            fontSize: findBestFontSize(cityName_1)),
                      ),
                    ),
                  ),
                  Container(
                    height: 51,
                    child: AnimatedContainer(
                      duration: Duration(milliseconds: 300),
                      curve: Curves.decelerate,
                      alignment: cityAlignment_2,
                      child: Text(
                        cityName_2,
                        style: TextStyle(
                            color: Color(0xff333333),
                            fontSize: findBestFontSize(cityName_2)),
                      ),
                    ),
                  ),
                  Container(
                    height: 51,
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: <Widget>[
                        InkWell(
                          onTap: () => _handleCityClicked(0),
                          child: SizedBox(
                            width: clickAreaW,
                            height: 51,
                          ),
                        ),
                        InkWell(
                          onTap: () => _handleCityExchange(state),
                          child: Container(
                            color: Colors.white,
                            padding: EdgeInsets.only(left: 2, right: 2),
                            child: Image.asset(
                              "images/icon_city_change.png",
                              package: "fflight",
                              width: 35.7,
                              height: 35.7,
                            ),
                          ),
                        ),
                        InkWell(
                          onTap: () => _handleCityClicked(1),
                          child: SizedBox(
                            width: clickAreaW,
                            height: 51,
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              );
            },
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              divider(
                  width: clickAreaW - 5.65,
                  height: 0.5,
                  color: Color(0xffe0e0e0)),
              divider(width: 51, height: 0.5, color: Color(0x00e0e0e0)),
              divider(
                  width: clickAreaW - 5.65,
                  height: 0.5,
                  color: Color(0xffe0e0e0)),
            ],
          ),

          /// 日期
          FSuper(
            width: double.infinity,
            height: 51,
            text: stringDateFormat(locationSearchDate, 'MM月dd日', isTwin: false),
            textAlignment: Alignment.centerLeft,
            textColor: Color(0xff202325),
            textSize: 21,
            spans: [
              FSuper.HorizontalSpace(6),
              TextSpan(
                  text: weekdayCN(getWeekday(locationSearchDate)),
                  style: TextStyle(
                    color: Color(0xff999999),
                    fontSize: 13,
                  ))
            ],
            child1: Icon(
              Icons.arrow_forward_ios,
              size: 12,
              color: Color(0xff999999),
            ),
            child1Alignment: Alignment.centerRight,
            onClick: _handlerDateClicked,
          ),
          Divider(
            height: 0.5,
            color: Color(0xffe0e0e0),
          ),
        ],
      ),
    );
  }

  double findBestFontSize(String cityName) {
    if (cityName.length > 8) {
      return 17;
    } else if (cityName.length > 6) {
      return 19;
    } else {
      return 21;
    }
  }

  void _handlerDateClicked() async {
    Map result = await FliggyNavigatorApi.getInstance().push(context, 'flight_lowprice_calendar', params: {
      'type': "0",
      'animeType': 1,
      'calendar_title': '出发日期',
      'begin_date': getBeforeDate(3),
      'date': _controller.index == 0?flightSearchDate:locationSearchDate,
      'dep_iata_code': depCityCode,
      'arr_iata_code': arrCityCode,
      'calendar_with_tab_header': false,
      //iOS参数
      'beginDate':_controller.index == 0?flightSearchDate:locationSearchDate,
      'validBeginDate':getBeforeDate(3),
    });
    setState(() {
      if (_controller.index == 0) {
        flightSearchDate = result['date'] ?? flightSearchDate;
      } else {
        locationSearchDate = result['date'] ?? locationSearchDate;
      }
    });
  }

  /// _controller.index == 0 按航班号查询，search_type = 1
  /// _controller.index == 1 按起降地查询，search_type = 2
  void _onSearchButtonClick() {
    Map args = {};
    if (_controller.index == 0) {
      if (textNotEmpty(flightNo)) {
        if (isAlphabetNumber(flightNo)) {
          track_click(context, this,
              ctrlName: FlightDynamicSpm.FLIGHT_NUM.name,
              args: args,
              ctrlSpm: FlightDynamicSpm.FLIGHT_NUM.spm);
          _commentFocus.unfocus();
          onSearchByFlightNum(flightNo, flightSearchDate);
        } else {
          _bridge.toast('亲，请输入正确的航班号');
        }
      } else {
        _bridge.toast('亲，请输入航班号再查询');
      }
    } else {
      if (cityExchanging) return;
      if (textIsEmpty(depCityName)) {
        _bridge.toast('亲，请输入出发城市');
        return;
      }
      if (textIsEmpty(arrCityName)) {
        _bridge.toast('亲，请输入到达城市');
        return;
      }
      if (depCityName == arrCityName) {
        _bridge.toast('亲，出发到达不能相同哦');
        return;
      }
      args["dep_city_name"] = depCityName;
      args["arr_city_name"] = arrCityName;
      args["dep_city_code"] = depCityCode;
      args["arr_city_code"] = arrCityCode;
      args["date"] = locationSearchDate;
      args["search_type"] = "2";
      track_click(context, this,
          ctrlName: FlightDynamicSpm.FLIGHT_CITY.name,
          args: args,
          ctrlSpm: FlightDynamicSpm.FLIGHT_CITY.spm);
      openDynamicListPage(args);
    }
  }

  void onSearchByFlightNum(String flightNo, String flightSearchDate) {
    FlightDynamicNet.requestListData<FlightDynamicListNetData>(
      flightNo: flightNo,
      flightDate: flightSearchDate,
      searchType: "1",
      pageIndex: 0,
      onFinish: (data) async {
        if (data != null && data.result != null && data.result.length == 1) {
          Map args = {};
          args["itemPos"] = 0;
          args["flightNo"] = flightNo;
          args["depDate"] = flightSearchDate;
          args["depAirportCode"] = data.result[0].depAirportCode;
          args["arrAirportCode"] = data.result[0].arrAirportCode;
          args["bizType"] = data.result[0].national ? 1 : 2;
          Map result = await openFlutterPage(context, "flight_dynamic_detail", args);
          _loginCallback();
        } else if (data != null &&
            data.result != null &&
            data.result.length > 1) {
          Map args = {};
          args["flight_number"] = flightNo;
          args["date"] = flightSearchDate;
          args["search_type"] = "1";
          openDynamicListPage(args);
        }
      },
      onFailed: (error) {
        Map args = {};
        args["flight_number"] = flightNo;
        args["date"] = flightSearchDate;
        args["search_type"] = "1";
        openDynamicListPage(args);
      },
      onEnd: () => hideFLoading(context),
    );
  }

  void openDynamicListPage(Map args) async {
    Map result = await openFlutterPage(context, "flight_dynamic_info_list", args);
    _loginCallback();
//    Navigator.push(context, MaterialPageRoute(
//      builder: (context) {
//        return FlightDynamicListPage(args);
//      },
//    ));
  }

  /// type = 0，出发城市
  /// type = 1，到达城市
  _handleCityClicked(int type) async {
    if (cityExchanging) return;
    Map args = {};
    args['biz_name'] = 'flight';
    args['biz_type'] = 'flight';//iOS参数
    args['fliterCountry'] = 0;//iOS参数
    args['biz_mode'] = 1;
    args['city_type'] = 0;
    args['animeType'] = 1;
    if (type == 0) {
      if (textNotEmpty(depCityName)) {
        args['bundle_current_city'] = depCityName;
        args['selected_city'] = depCityName;
        args['iata_code'] = depCityCode;//iOS参数
      }
    } else {
      if (textNotEmpty(arrCityName)) {
        args['bundle_current_city'] = arrCityName;
        args['selected_city'] = arrCityName;
        args['iata_code'] = arrCityCode;//iOS参数
      }
    }
    Map result = await FliggyNavigatorApi.getInstance().push(context, 'common_city_select', params: args);
    print("result = ${result}");
    if (Platform.isIOS) {
      Map city = result['city'];
      if (city != null && city.isNotEmpty) {
        if (type == 0) {
          setState(() {
            depCityName = city['city_name'];
            depCityCode = city['iata_code'];
          });
        } else {
          setState(() {
            arrCityName = city['city_name'];
            arrCityCode = city['iata_code'];
          });
        }
      }
    } else if(result is Map){
      if (type == 0) {
        setState(() {
          depCityName = result['city_name'] ?? depCityName;
          depCityCode = result['iata_code'] ?? depCityCode;
        });
      } else {
        setState(() {
          arrCityName = result['city_name'] ?? arrCityName;
          arrCityCode = result['iata_code'] ?? arrCityCode;
        });
      }
    }
  }

  void _handleFlightNumberClearClicked() {
    _inputFlightNoController.text = '';
    flightNo = "";
  }

  _handleCityExchange(StateSetter state) {
    if (cityExchanging) return;
    state(() {
      cityExchanging = true;
      if (cityAlignment_1 == Alignment.centerLeft) {
        cityAlignment_1 = Alignment.centerRight;
        cityAlignment_2 = Alignment.centerLeft;
      } else {
        cityAlignment_1 = Alignment.centerLeft;
        cityAlignment_2 = Alignment.centerRight;
      }
      String temp = depCityName;
      depCityName = arrCityName;
      arrCityName = temp;
      temp = depCityCode;
      depCityCode = arrCityCode;
      arrCityCode = temp;
    });
  }

  requestData([StateSetter state]) {
    if (!hasLogin()) return;
    FlightDynamicNet.requestAttentionListData<FlightDynamicListNetData>(
      onFinish: (data) {
        if (state != null) {
          state(() {
            datas = data?.result;
          });
        } else {
          setState(() {
            datas = data?.result;
          });
        }
      },
      onFailed: (error) {
        if (state != null) {
          state(() {
            datas = null;
          });
        } else {
          setState(() {
            datas = null;
          });
        }
      },
    );
  }

  _loginCallback() {
    debugPrint("_loginCallback");
    checkUser(setState);
  }

  @override
  String getPageName() {
    return "flight_dynamic_home";
  }

  @override
  String getPageSpmCnt() {
    return "181.11332725.0.0";
  }
}

class WhitelistingTextInputFormatterWithUppercase extends TextInputFormatter {
  /// Creates a formatter that allows only the insertion of whitelisted characters patterns.
  ///
  /// The [whitelistedPattern] must not be null.
  WhitelistingTextInputFormatterWithUppercase(this.whitelistedPattern)
      : assert(whitelistedPattern != null);

  /// A [Pattern] to extract all instances of allowed characters.
  ///
  /// [RegExp] with multiple groups is not supported.
  final Pattern whitelistedPattern;

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, // unused.
      TextEditingValue newValue,) {
    return _selectionAwareTextManipulation(
      newValue,
          (String substring) {
        return whitelistedPattern
            .allMatches(substring)
            .map<String>((Match match) => match.group(0))
            .join()
            .toUpperCase();
      },
    );
  }

  /// A [WhitelistingTextInputFormatter] that takes in digits `[0-9]` only.
  static final WhitelistingTextInputFormatter digitsOnly =
  WhitelistingTextInputFormatter(RegExp(r'\d+'));
}

TextEditingValue _selectionAwareTextManipulation(TextEditingValue value,
    String substringManipulation(String substring),) {
  final int selectionStartIndex = value.selection.start;
  final int selectionEndIndex = value.selection.end;
  String manipulatedText;
  TextSelection manipulatedSelection;
  if (selectionStartIndex < 0 || selectionEndIndex < 0) {
    manipulatedText = substringManipulation(value.text);
  } else {
    final String beforeSelection =
    substringManipulation(value.text.substring(0, selectionStartIndex));
    final String inSelection = substringManipulation(
        value.text.substring(selectionStartIndex, selectionEndIndex));
    final String afterSelection =
    substringManipulation(value.text.substring(selectionEndIndex));
    manipulatedText = beforeSelection + inSelection + afterSelection;
    if (value.selection.baseOffset > value.selection.extentOffset) {
      manipulatedSelection = value.selection.copyWith(
        baseOffset: beforeSelection.length + inSelection.length,
        extentOffset: beforeSelection.length,
      );
    } else {
      manipulatedSelection = value.selection.copyWith(
        baseOffset: beforeSelection.length,
        extentOffset: beforeSelection.length + inSelection.length,
      );
    }
  }
  return TextEditingValue(
    text: manipulatedText,
    selection:
    manipulatedSelection ?? const TextSelection.collapsed(offset: -1),
    composing:
    manipulatedText == value.text ? value.composing : TextRange.empty,
  );
}
