import 'dart:convert';

import 'package:fbridge/fbridge.dart';
import 'package:fflight/common/base_page.dart';
import 'package:fflight/common/date_utils.dart';
import 'package:fflight/common/tracker.dart';
import 'package:fflight/common/utils.dart';
import 'package:fflight/common/widget_utils.dart';
import 'package:fflight/page/flightdynamic/flight_dynamic_spm.dart';
import 'package:fflight/page/flightdynamic/net/flight_dynamic_net.dart';
import 'package:fflight/widget/flightdynamiclistcell/flight_dynamic_list_cell_data.dart';
import 'package:fflight/widget/flightdynamiclistcell/flight_dynamic_list_cell_view.dart';
import 'package:fflight/widget/fresult/fresult.dart';
import 'package:fflight/widget/fsuper/fsuper.dart';
import 'package:flutter/material.dart';
import 'package:fliggy_router/fliggy_router.dart';
import 'package:ftitlebar_adapter/ftitlebar_adapter.dart';
import 'package:ftitlebar/ftitlebar.dart';
import 'net/flight_dynamic_list_net_data.dart';
import 'net/flight_follow_data.dart';

class FlightDynamicListPage extends StatefulWidget {
  Map args;

  FlightDynamicListPage(this.args);

  @override
  _FlightDynamicListPageState createState() => _FlightDynamicListPageState();
}

class _FlightDynamicListPageState extends State<FlightDynamicListPage>
    with PageTracker {

  ScrollController _listController = new ScrollController();

  String requestPageCode;
  String date;
  String depCity;
  String arrCity;
  String depCityCode;
  String arrCityCode;
  String flightNumber;

  /// 查询类型 1=按航班号查询、2=按航线查询
  String searchType;

  int pageIndex = 0;
  bool hasMore = false;

  List<FlightDynamicListCellData> datas;

  Widget errorView;

  bool isFirstBuild = true;

  FBridge _bridge;
  
  @override
  void initState() {
    super.initState();
    _bridge = FBridgeApi.newInstance(context);
    track_page(context, this);
    _listController.addListener(() {
      if (_listController.position.pixels ==
              _listController.position.maxScrollExtent &&
          hasMore) {
        requestData();
      }
    });
  }

  void parseArgs() {
    if (widget.args == null) return;
    flightNumber = widget.args["flight_number"];
    date = widget.args["date"];
    depCity = widget.args["dep_city_name"];
    arrCity = widget.args["arr_city_name"];
    depCityCode = widget.args["dep_city_code"];
    arrCityCode = widget.args["arr_city_code"];
    searchType = widget.args["search_type"];
  }

  @override
  Widget build(BuildContext context) {
    if (isFirstBuild) {
      isFirstBuild = false;
      dynamic obj = ModalRoute.of(context).settings.arguments;
      if (obj != null && obj is Map && obj.isNotEmpty) {
        widget.args = obj;
      }
      parseArgs();
      requestData();
    }
    List<Widget> pageChildren = _buildPageChildren();
    return Scaffold(
      appBar: FTitleBar(
        title: _generateTitle(),
        hasMore: true,
        themeUpdateCallback: FTitleBarAdapter.themeUpdateCallback,
        moreChildren: FTitleBarAdapter.getPopupMenuItemData(),
        onMorePressed: FTitleBarAdapter.getPopupMenuPressedCallback(context),
      ),
      body: Container(
        color: Color(0xfff4f4f4),
        child: Column(
          children: pageChildren,
        ),
      ),
    );
  }

  List<Widget> _buildPageChildren() {
    var pageChildren = <Widget>[];

    /// header
    var headerPart = _buildHeader();
    pageChildren.add(headerPart);

    if (errorView != null) {
      pageIndex = 0;
      hasMore = false;
      var errorPart = Container(
          margin: EdgeInsets.only(
            top: 152.5,
          ),
          child: errorView);
      pageChildren.add(errorPart);
    } else {
      /// 提示
      var tipsPart = FSuper(
        width: double.infinity,
        backgroundColor: Colors.white,
        text: '航班起降时间均为当地时间',
        textColor: Color(0xff666666),
        textSize: 13,
        padding: EdgeInsets.only(top: 6, bottom: 6),
        margin: EdgeInsets.only(bottom: 0.5),
      );
      pageChildren.add(tipsPart);

      /// list
      if (datas != null) {
        var listPart = Expanded(
          child: ListView.builder(
            itemCount: datas.length + 1,
            itemBuilder: _buildListCell,
            physics: BouncingScrollPhysics(),
            controller: _listController,
          ),
        );
        pageChildren.add(listPart);
      }
    }
    return pageChildren;
  }

  String _generateTitle() {
    if (textNotEmpty(depCity) && textNotEmpty(arrCity)) {
      return '$depCity - $arrCity';
    } else {
      return safeString(flightNumber);
    }
  }

  Container _buildHeader() {
    bool noBeforeDay = daysBetween(DateTime.parse(date), DateTime.now().subtract(Duration(days: 3)), true) <= 0;
    return Container(
      padding: EdgeInsets.only(left: 12, right: 12),
      height: 40,
      decoration: BoxDecoration(
          gradient: LinearGradient(colors: [
        Color(0xffffd800),
        Color(0xffffc900),
      ])),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          FSuper(
            height: 40,
            text: "前一天",
            textColor: noBeforeDay ? Color(0xff999999) : Color(0xff3d3d3d),
            textSize: 13,
            textAlignment: Alignment.centerLeft,
            padding: EdgeInsets.only(left: 15, right: 6),
            child1: Icon(
              Icons.arrow_back_ios,
              size: 12,
              color: noBeforeDay ? Color(0xff999999) : Color(0xff3d3d3d),
            ),
            child1Alignment: Alignment.centerLeft,
            onClick: noBeforeDay ? null : _handleTheDayBeforeClicked,
          ),
          FSuper(
            width: 149,
            height: 29,
            text:
                '${stringDateFormat(date, 'MM月dd日')} ${weekdayCN(getWeekday(date))}',
            textColor: Color(0xff3d3d3d),
            textSize: 13,
            textAlignment: Alignment.centerLeft,
            padding: EdgeInsets.only(left: 15),
            child1: Row(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                verticalDivider(
                  height: 23,
                  width: 1,
                  color: Color(0xff3d3d3d),
                ),
                const SizedBox(
                  width: 9,
                ),
                images(
                  "icon_rili.png",
                  width: 15,
                  height: 15,
                ),
              ],
            ),
            child1Alignment: Alignment.centerRight,
            child1Margin: EdgeInsets.only(right: 9),
            corner: Corner.all(3),
            backgroundColor: Colors.white,
            onClick: _handlerDateClicked,
          ),
          FSuper(
            height: 40,
            text: "后一天",
            textColor: Color(0xff3d3d3d),
            textSize: 13,
            padding: EdgeInsets.only(left: 6, right: 15),
            textAlignment: Alignment.centerRight,
            child1: Icon(
              Icons.arrow_forward_ios,
              size: 12,
              color: Color(0xff3d3d3d),
            ),
            child1Alignment: Alignment.centerRight,
            onClick: _handleTheDayAfterClicked,
          ),
        ],
      ),
    );
  }

  Widget _buildListCell(BuildContext context, int index) {
    var cell;
    if (index != datas.length) {
      cell = StatefulBuilder(builder: (context, state) {
        return InkWell(
          onTap: () async {
            FlightDynamicListCellData data = datas[index];
            Map args = {
              'itemPos': index,
              'flightNo': data.flightNo,
              'depDate': date,
              'depAirportCode': data.depAirportCode,
              'arrAirportCode': data.arrAirportCode,
              'bizType': data.national ? 1 : 2,
            };
            Map result = await openFlutterPage(context, "flight_dynamic_detail", args);
            setState(() {
              if (result != null && result.isNotEmpty) {
                datas[result['itemPos']].attention = result['attention'] ?? false;
              }
            });
//            await Navigator.push(context, MaterialPageRoute(
//              builder: (context) {
//
//                return FlightDynamicDetailPage(args);
//              },
//            )).then((result) {
//              setState(() {
//                datas[index].attention = result ?? false;
//              });
//            });
          },
          child: Padding(
            padding: const EdgeInsets.only(bottom: 6),
            child: FlightDynamicListCellView(
              datas[index],
              showCity: searchType == '1',
            )..attentionClicked = () => _handleAttentionClicked(index, state),
          ),
        );
      });
    } else {
      cell = Container(
        margin: const EdgeInsets.only(top: 12, bottom: 20),
        alignment: Alignment.center,
        child: Text(
          hasMore ? '正在加载..' : '没有更多了',
          style: TextStyle(
            color: Color(0xffcccccc),
            fontSize: 12,
          ),
        ),
      );
    }
    return cell;
  }

  void _handlerDateClicked() async {
    Map result = await FliggyNavigatorApi.getInstance().push(context, 'flight_lowprice_calendar', params: {
      'type': 0,
      'animeType': 1,
      'calendar_title': '出发日期',
      'begin_date': getBeforeDate(3),
      'date': date,
      'dep_iata_code': depCityCode,
      'arr_iata_code': arrCityCode,
      'calendar_with_tab_header': false,
      //iOS参数
      'beginDate':date,
      'validBeginDate':getBeforeDate(3),
    });
    if (result['date'] != null && result['date'] != date) {
      updateDate(result['date']);
    }
  }

  void _handleTheDayBeforeClicked() {
    setState(() {
      var newDate = DateTime.parse(date).subtract(Duration(days: 1));
      updateDate(dateFormat(newDate, 'yyyy-MM-dd'));
    });
  }

  void _handleTheDayAfterClicked() {
    var newDate = DateTime.parse(date).add(Duration(days: 1));
    updateDate(dateFormat(newDate, 'yyyy-MM-dd'));
  }

  updateDate(String date) {
    setState(() {
      pageIndex = 0;
      this.date = date;
      datas = null;
      requestData();
    });
  }

  requestData() {
    showFLoading(context);
    final String requestDate = date;
    FlightDynamicNet.requestListData<FlightDynamicListNetData>(
      flightNo: flightNumber,
      flightDate: date,
      depCityCode: depCityCode,
      arrCityCode: arrCityCode,
      searchType: searchType ?? '1',
      pageIndex: pageIndex,
      onFinish: (data) {
        if (requestDate != date) return;
        setState(() {
          if (data != null && data.result != null && data.result.length != 0) {
            errorView = null;
            if (pageIndex == 0) {
              datas = List();
            }
            datas.addAll(data.result);
            pageIndex += 1;
            hasMore = data.hasMore;
          } else {
            setState(() {
              errorView = FResult.noGoods(
                'widget/fresult/',
                'fflight',
                onButtonPress: () {},
                errorText: "亲，当前无航班数据",
//                showViceError: false,
//                showBtnContainer: false,
              );
            });
          }
        });
      },
      onFailed: (error) {
        if (requestDate != date || pageIndex != 0) return;
        switch (error['errorMsg']) {
          case 'ERROR_MSG_NET_ERROR':
            setState(() {
              errorView = FResult.noNetwork('widget/fresult/', 'fflight',
                  onButtonPress: () {
                pageIndex = 0;
                requestData();
              }, onViceBtnPress: null);
            });
            break;
          default:
            setState(() {
              errorView = FResult.noGoods(
                'widget/fresult/',
                'fflight',
                onButtonPress: () {},
                errorText: error['errorDesc'],
//                showViceError: false,
//                showBtnContainer: false,
              );
            });
        }
      },
      onEnd: () => hideFLoading(context),
    );
  }

  void _handleAttentionClicked(int index, StateSetter state) async {
    showFLoading(context);
    FlightDynamicListCellData cellData = datas[index];
    var attendType = cellData.attention ? '-1' : '1';
    if (attendType == "1") {
      track_click(context, this,
          ctrlName: FlightDynamicSpm.Add_attention_list.name,
          ctrlSpm: FlightDynamicSpm.Add_attention_list.spm);
    } else {
      track_click(context, this,
          ctrlName: FlightDynamicSpm.Cancelled_attention_list.name,
          ctrlSpm: FlightDynamicSpm.Cancelled_attention_list.spm);
    }
    FlightDynamicNet.requestAttention<FlightFollowData>(
      flightNo: cellData.flightNo,
      flightDate: date,
      depAirportCode: cellData.depAirportCode,
      arrAirportCode: cellData.arrAirportCode,
      attendType: attendType,
      onFinish: (data) {
        if (data != null && !data.failure) {
          state(() {
            cellData.attention = attendType == '1';
            if (cellData.attention) {
              _bridge.toast('关注成功');
            } else {
              _bridge.toast('取消关注成功');
            }
          });
        }
      },
      onFailed: (error) {
        if(error is Map && error['errorDesc'] != "system error"){
          _bridge.toast(error['errorDesc']);
        }
      },
      onEnd: () => hideFLoading(context),
    );
  }

  @override
  String getPageName() {
    return "flight_dynamic_info_list";
  }

  @override
  String getPageSpmCnt() {
    return "181.11332736.0.0";
  }
  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
      track_page(context, this);
    }
  }
}
