import 'dart:ui';

import 'package:fbridge/fbridge.dart';
import 'package:fflight/common/base_page.dart';
import 'package:fflight/common/tracker.dart';
import 'package:fflight/common/utils.dart';
import 'package:fflight/page/flightdynamic/net/flight_dynamic_locus_net_data.dart';
import 'package:fflight/widget/fresult/fresult.dart';
import 'package:flutter/material.dart';
import 'package:fliggy_router/fliggy_router.dart';
import 'package:ftitlebar_adapter/ftitlebar_adapter.dart';
import 'package:ftitlebar/ftitlebar.dart';
import 'net/flight_dynamic_net.dart';

class FlightDynamicLocusPage extends StatefulWidget {
  Map args;

  FlightDynamicLocusPage(this.args);

  @override
  State<StatefulWidget> createState() {
    // TODO: implement createState
    return _FlightDynamicLocusPageState();
  }
}

Widget errorView;

class _FlightDynamicLocusPageState extends State<FlightDynamicLocusPage> with PageTracker{

  String flightNo;
  String depDate;
  String depAirportCode;
  String arrAirportCode;
  String orderId;
  /// 1-国内；2-国际
  int bizType;
  List<FlightDynamicLocusCellData> datas;
  bool isFirstBuild = true;
  FBridge _bridge;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    _bridge = FBridgeApi.newInstance(context);
    track_page(context, this);
  }

  @override
  Widget build(BuildContext context) {
    if (isFirstBuild) {
      isFirstBuild = false;
      dynamic obj = ModalRoute.of(context).settings.arguments;
      if (obj != null && obj is Map && obj.isNotEmpty) {
        widget.args = obj;
      }
      parseArgs();
      _requestData();
    }
    var children = <Widget>[];
    _addBackgroundPart(children);
    children.add(_buildMainContent());

    return WillPopScope(
      child: Scaffold(
        appBar: FTitleBar(
          title: '航班跟踪',
          hasMore: true,
          themeUpdateCallback: FTitleBarAdapter.themeUpdateCallback,
          moreChildren: FTitleBarAdapter.getPopupMenuItemData(),
          onMorePressed: FTitleBarAdapter.getPopupMenuPressedCallback(context),
        ),
        body: Container(
            alignment: Alignment.center,
            color: Color(0xfff4f4f4),
            child: errorView == null
                ? Stack(
                    children: children,
                  )
                : Padding(
                    padding: EdgeInsets.only(top: 165),
                    child: errorView,
                  )),
      ),
      // ignore: missing_return
      onWillPop: (){
        track_click(context, this, ctrlName: "Back");
        FliggyNavigatorApi.getInstance().pop(context);
        return new Future.value(false);
      },
    );
  }
  Widget _buildListCell(BuildContext context, int index){
    var cell;
    if(datas!=null&&index!=datas.length){
      FlightDynamicLocusCellData data=datas[index];
      String date=data.showData.createTimeDes;
      if(date!=null&&date.contains("-")){
        List<String>split = date.split(" ");
        if (split.length > 0) {
          String tempDate = split[0];
          tempDate = tempDate.substring(5);
          if (tempDate.startsWith("0")) {
            tempDate = tempDate.substring(1);
          }
          tempDate = tempDate.replaceAll("-", "月");
          tempDate = tempDate + "日";
          data.showData.createTimeDes=tempDate + date.substring(10);
        }
      }
      if(index==datas.length-1){
        cell=_buildTimeLine(data.showData.createTimeDes,data.showData.createTimeColor,data.showData.trackerTitleDes,data.showData.trackerTitleColor,data.showData.trackerRemarkDes,data.showData.trackerRemarkColor,index,true);
      }else{
        cell=_buildTimeLine(data.showData.createTimeDes,data.showData.createTimeColor,data.showData.trackerTitleDes,data.showData.trackerTitleColor,data.showData.trackerRemarkDes,data.showData.trackerRemarkColor,index,false);
      }
    }
    if(cell==null){
      return EMPTY_WIDGET;
    }
    return cell;
  }

  Widget _buildMainContent() {
    return Container(
      margin: EdgeInsets.only(
        top:30.0
      ),

        child: ListView.builder(
          itemCount: datas!=null?datas.length:0,
          itemBuilder: _buildListCell,
          physics: BouncingScrollPhysics(),
        ),
//      child: ListView(
//        children: <Widget>[
//          if(datas!=null){
//
//          }
//          _buildTimeLine('3月2日 15:55 当地','登机结束','踢足球', 0,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','玩游戏', 1,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','看比赛', 2,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','打篮球', 3,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','玩游戏', 4,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','看比赛', 5,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','打篮球', 6,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','看比赛', 2,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','打篮球', 3,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','玩游戏', 4,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','看比赛', 5,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','打篮球', 6,false),
//          _buildTimeLine('3月2日 15:55 当地','登机结束','休息', 7,true),
//        ],
//      ),
    );
  }


  void parseArgs() {
    if (widget.args == null) return;
    flightNo = widget.args["flightNo"];
    depDate = widget.args["depDate"];
    depAirportCode = widget.args["depAirportCode"];
    arrAirportCode = widget.args["arrAirportCode"];
    bizType = widget.args["bizType"];
    orderId=widget.args["orderId"];
  }

  void _addBackgroundPart(List<Widget> children) {
    children.add(Container(
      height: 250,
      decoration: BoxDecoration(
          gradient: LinearGradient(
        colors: [
          Color(0xffffd800),
          Color(0xffffc900),
        ],
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
      )),
    ));
    children.add(Container(
      height: 250,
      decoration: BoxDecoration(
          gradient: LinearGradient(
        colors: [
          Color(0x00f4f4f4),
          Color(0xfff4f4f4),
        ],
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
      )),
    ));
  }

  Widget _buildTimeLine(String time,String timeColor,String title,String titleColor,String message,String messageColor, int pos,bool las) {
    return Stack(
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.only(left: 40.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Container(
                margin: const EdgeInsets.only(left: 35.0),
                child: Text(time,style: new TextStyle(color:textNotEmpty(timeColor)?ADColor(timeColor):Color(0xFF888888),fontSize: 14.0),)),
              Card(
                margin: EdgeInsets.only(
                    left: 20.0, top: 4.0, right: 20.0, bottom: 10.0),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Container(
                      padding: EdgeInsets.only(left: 15.0,right: 16.0,top: 10.0,bottom: 5.0),
                      width: double.infinity,
                      child: Text(title,style: new TextStyle(color: textNotEmpty(titleColor)?ADColor(titleColor):Color(0xFF66BD5C),fontSize: 18.0)),
                    ),
                    Container(
                      padding: EdgeInsets.only(left: 16.0,right: 16.0,top: 5.0,bottom: 10.0),
                      width: double.infinity,
                      child: Text(message,style: new TextStyle(color: textNotEmpty(messageColor)?ADColor(messageColor):Color(0xFF666666),fontSize: 14.0)),
                    )
                  ],
                ),
              ),
            ],
          ),
        ),
        Positioned(
          top: pos==0?45.0:0.0,
          bottom:las==true ? 50.0:0.0,
          left: 36.5,
          child: Container(

            height: double.infinity,
            width: 2.0,
            color: Color(0xFF888888),
          ),
        ),
        Positioned(
          top: 13.0,
          left: 20,
          child: pos == 0
              ? Container(
                  margin: EdgeInsets.only(top: 30.0),
                  child: Image.asset(
                    "images/icon_flight_fly.png",
                    package: "fflight",
                    width: 35,
                    height: 35,
                  ))
              : Container(
                  margin: EdgeInsets.only(left: 11.5, top: 45.0),
                  height: 12.0,
                  width: 12.0,
                  decoration: BoxDecoration(
                      shape: BoxShape.circle, color: Color(0xFF888888)),
                ),
        )
      ],
    );
  }


  _requestData() {
    showFLoading(context);
    FlightDynamicNet.requestDynamicLocus<FlightDynamicLocusNetData>(
      flightNo: flightNo,
      flightDate: depDate,
      depAirportCode: depAirportCode,
      arrAirportCode: arrAirportCode,
      bizType: bizType?.toString(),
      orderId: orderId,
      onFinish: (data) {
        if (data != null && data.result != null) {
          errorView = null;
          setState(() {
            this.datas = data.result;
          });
        } else {
          this.datas = null;
          setState(() {
            errorView = FResult.noGoods(
              'widget/fresult/',
              'fflight',
              onButtonPress: () {},
              errorText: "亲，当前无航班轨迹",
//              showViceError: false,
//              showBtnContainer: false,
            );
          });
        }
      },
      onFailed: (error) {
        switch (error['errorMsg']) {
          case 'ERROR_MSG_NET_ERROR':
            setState(() {
              errorView = FResult.noNetwork('widget/fresult/', 'fflight',
                  onButtonPress: () {
                    _requestData();
                  }, onViceBtnPress: null);
            });
            break;
          default:
            setState(() {
              errorView = FResult.noGoods(
                'widget/fresult/',
                'fflight',
                onButtonPress: () {},
                errorText: error['errorDesc'],
//                showViceError: false,
//                showBtnContainer: false,
              );
            });
        }
      },
      onEnd: () {
        hideFLoading(context);
      },
    );
  }

  Color ADColor(String colorString, {double alpha = 1.0}) {
    String colorStr = colorString;
    // colorString未带0xff前缀并且长度为6
    if (!colorStr.startsWith('0xff') && colorStr.length == 6) {
      colorStr = '0xff' + colorStr;
    }
    // colorString为8位，如0x000000
    if(colorStr.startsWith('0x') && colorStr.length == 8) {
      colorStr = colorStr.replaceRange(0, 2, '0xff');
    }
    // colorString为7位，如#000000
    if(colorStr.startsWith('#') && colorStr.length == 7) {
      colorStr = colorStr.replaceRange(0, 1, '0xff');
    }
    // 先分别获取色值的RGB通道
    Color color = Color(int.parse(colorStr));
    int red = color.red;
    int green = color.green;
    int blue = color.blue;
    // 通过fromRGBO返回带透明度和RGB值的颜色
    return Color.fromRGBO(red, green, blue, alpha);
  }

  @override
  String getPageName() {
    return "flight_dynamic_info_trajectory";
  }

  @override
  String getPageSpmCnt() {
    return "181.11708117.0.0";
  }
  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
        track_page(context, this);
    }
  }
}
