import 'package:fflight/common/net.dart';
import 'package:fflight/common/utils.dart';
import 'package:flutter/cupertino.dart';
import 'flight_dynamic_detail_net_data.dart';
import 'flight_dynamic_list_net_data.dart';
import 'flight_dynamic_locus_net_data.dart';
import 'flight_follow_data.dart';

class FlightDynamicNet {
  /// 航班动态列表页列表接口
  static requestListData<T>({
    BuildContext context,
    VoidCallback onStart,
    NetCallback<T> onFinish,
    NetErrorCallback onFailed,
    VoidCallback onEnd,
    String flightNo,
    String flightDate,
    String depCityCode,
    String arrCityCode,
    String searchType,
    int pageIndex,
  }) {
    Map params = {};
    if (textNotEmpty(flightNo)) params['flightNo'] = flightNo;
    if (textNotEmpty(flightDate)) params['flightDate'] = flightDate;
    if (textNotEmpty(depCityCode)) params['depCityCode'] = depCityCode;
    if (textNotEmpty(arrCityCode)) params['arrCityCode'] = arrCityCode;
    if (textNotEmpty(searchType)) params['searchType'] = searchType;
    if (pageIndex != null) params['pageIndex'] = pageIndex;
    net<T>(
      context,
      'mtop.alitirp.atus.flight.dynamic.search',
      '1.0',
      params,
      FlightDynamicListNetData.fromMap,
      onStart: onStart,
      onFinish: onFinish,
      onFailed: onFailed,
      onEnd: onEnd,
    );
  }

  /// 航班动态首页关注列表
  static requestAttentionListData<T>({
    BuildContext context,
    VoidCallback onStart,
    NetCallback<T> onFinish,
    NetErrorCallback onFailed,
    VoidCallback onEnd,
  }) {
    Map params = {};
    net<T>(
      context,
      'mtop.alitirp.atus.flight.attention.search',
      '1.0',
      params,
      FlightDynamicListNetData.fromMap,
      needLogin: true,
      onStart: onStart,
      onFinish: onFinish,
      onFailed: onFailed,
      onEnd: onEnd,
    );
  }

  /// 航班动态关注接口
  static void requestAttention<T>({
    BuildContext context,
    VoidCallback onStart,
    NetCallback<T> onFinish,
    NetErrorCallback onFailed,
    VoidCallback onEnd,
    String flightNo,
    String flightDate,
    String depAirportCode,
    String arrAirportCode,
    /// 1-关注；-1-取消关注
    String attendType,
  }) {
    Map params = {};
    if (textNotEmpty(flightNo)) params['flightNo'] = flightNo;
    if (textNotEmpty(flightDate)) params['flightDate'] = flightDate;
    if (textNotEmpty(depAirportCode)) params['depAirportCode'] = depAirportCode;
    if (textNotEmpty(arrAirportCode)) params['arrAirportCode'] = arrAirportCode;
    if (textNotEmpty(attendType)) params['attendType'] = attendType;
    net<T>(
      context,
      'mtop.alitirp.atus.flight.attention.attend',
      '1.0',
      params,
      FlightFollowData.fromMap,
      needLogin: true,
      onStart: onStart,
      onFinish: onFinish,
      onFailed: onFailed,
      onEnd: onEnd,
    );
  }

  /// 航班动态详情接口
  static void requestDynamicDetail<T>({
    BuildContext context,
    VoidCallback onStart,
    NetCallback<T> onFinish,
    NetErrorCallback onFailed,
    VoidCallback onEnd,
    String flightNo,
    String flightDate,
    String depAirportCode,
    String arrAirportCode,
    String bizType,
  }) {
    Map params = {};
    if (textNotEmpty(flightNo)) params['flightNo'] = flightNo;
    if (textNotEmpty(flightDate)) params['flightDate'] = flightDate;
    if (textNotEmpty(depAirportCode)) params['depAirportCode'] = depAirportCode;
    if (textNotEmpty(arrAirportCode)) params['arrAirportCode'] = arrAirportCode;
    if (textNotEmpty(bizType)) params['bizType'] = bizType;
    net<T>(
      context,
      'mtop.alitirp.atus.flight.dynamic.detail',
      '1.0',
      params,
      FlightDynamicDetailNetData.fromMap,
      onStart: onStart,
      onFinish: onFinish,
      onFailed: onFailed,
      onEnd: onEnd,
    );
  }

  /// 航班轨迹
  static void requestDynamicLocus<T>({
    BuildContext context,
    VoidCallback onStart,
    NetCallback<T> onFinish,
    NetErrorCallback onFailed,
    VoidCallback onEnd,
    String flightNo,
    String flightDate,
    String depAirportCode,
    String arrAirportCode,
    String orderId,
    String bizType,
  }) {
    Map params = {};
    if (textNotEmpty(flightNo)) params['flightNo'] = flightNo;
    if (textNotEmpty(flightDate)) params['flightDate'] = flightDate;
    if (textNotEmpty(depAirportCode)) params['depAirportCode'] = depAirportCode;
    if (textNotEmpty(arrAirportCode)) params['arrAirportCode'] = arrAirportCode;
    if (textNotEmpty(orderId)) params['orderId'] = orderId;
    if (textNotEmpty(bizType)) params['bizType'] = bizType;
    net<T>(
      context,
      'mtop.alitirp.atus.flight.tracker.search',
      '1.0',
      params,
      FlightDynamicLocusNetData.fromMap,
      onStart: onStart,
      onFinish: onFinish,
      onFailed: onFailed,
      onEnd: onEnd,
    );
  }
}
