import 'package:fflight/common/date_utils.dart';
import 'package:fflight/common/utils.dart';
import 'package:fflight/widget/fsuper/fsuper.dart';
import 'package:flutter/material.dart';
import 'flight_dynamic_list_cell_data.dart';

class FlightDynamicListCellView extends StatefulWidget {
  FlightDynamicListCellData data;

  VoidCallback attentionClicked;

  bool showAttention;
  bool showDate;
  bool showCity;

  FlightDynamicListCellView(
    this.data, {
    this.showAttention = true,
    this.showDate = false,
    this.showCity = true,
  });

  @override
  _FlightDynamicListCellViewState createState() =>
      _FlightDynamicListCellViewState();
}

class _FlightDynamicListCellViewState extends State<FlightDynamicListCellView> {
  @override
  Widget build(BuildContext context) {
    if (widget.data == null) return Container();
    Widget child;
    if (widget.showAttention) {
      child = IntrinsicHeight(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: <Widget>[
            _buildFlightInfoPart(),
            _buildAttentionPart(),
          ],
        ),
      );
    } else {
      child = Row(
        children: <Widget>[
          _buildFlightInfoPart(),
        ],
      );
    }
    return Container(
      color: Colors.white,
      child: child,
    );
  }

  Widget _buildFlightInfoPart() {
    List<Widget> children = [];
    var flightNumberPart = widget.data.airlineIcon != null
        ? FSuper(
            margin: EdgeInsets.only(top: 10, bottom: 6),
            textAlignment: Alignment.centerLeft,
            textColor: Color(0xff999999),
            textSize: 13,
            text:
                '${safeString(widget.data.airlineName)} ${safeString(widget.data.flightNo)}',
            padding: EdgeInsets.only(left: 18),
            child1:
                Image.network(widget.data.airlineIcon, width: 15, height: 15),
            child1Alignment: Alignment.centerLeft,
          )
        : FSuper(
            margin: EdgeInsets.only(top: 13, bottom: 10.6),
            textAlignment: Alignment.centerLeft,
            textColor: Color(0xff999999),
            textSize: 13,
            text:
                '${safeString(widget.data.airlineName)} ${safeString(widget.data.flightNo)}',
          );
    children.add(flightNumberPart);
    if (widget.showCity) {
      var cityPart = _buildCityPart();
      children.add(Padding(
        padding: const EdgeInsets.only(bottom: 6),
        child: cityPart,
      ));
    }
    var timePart = _buildTimePart();
    children.add(
      Padding(
        padding: const EdgeInsets.only(bottom: 1),
        child: timePart,
      ),
    );
    var airportPart = _buildAirportPart();
    children.add(Padding(
      padding: const EdgeInsets.only(bottom: 12),
      child: airportPart,
    ));
    return Expanded(
      child: Padding(
        padding: const EdgeInsets.only(left: 12, right: 12),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: children,
        ),
      ),
    );
  }

  Widget _buildTimePart() {
    var arrDateTime = DateTime.parse(widget.data.arrTime);
    var arrDatePart =
        daysBetween(arrDateTime, DateTime.parse(widget.data.depTime)) > 0
            ? Text(
                '${dateFormat(arrDateTime, 'MM-dd')}\n${weekdayCN(arrDateTime.weekday)}',
                textAlign: TextAlign.right,
                style: TextStyle(
                  color: Color(0xff999999),
                  fontSize: 10,
                  height: 1,
                ),
              )
            : null;
    var depTime =
        widget.data.depTime == null ? '' : widget.data.depTime.split(" ")[1];
    var arrTime =
        widget.data.arrTime == null ? '' : widget.data.arrTime.split(" ")[1];
    var stateColor =
        color(widget.data.showData.flightStatusColor, defaultColor: '#2ea3ff');
    return Stack(
      alignment: Alignment.center,
      children: <Widget>[
        Align(
          alignment: Alignment.centerLeft,
          child: FSuper(
            width: 106,
            text: depTime,
            textColor: Color(0xff333333),
            textSize: 24,
            textAlign: TextAlign.left,
          ),
        ),
        FSuper(
          width: 74,
          height: 27,
          corner: Corner.all(3),
          strokeWidth: 1,
          strokeColor: stateColor,
          text: safeString(widget.data.flightStatus),
          textColor: stateColor,
          textAlignment: Alignment.center,
        ),
        Align(
          alignment: Alignment.centerRight,
          child: FSuper(
            width: 106,
            text: arrTime,
            textColor: Color(0xff333333),
            textSize: 24,
            textAlign: TextAlign.right,
            child1: arrDatePart,
            child1Alignment: Alignment.centerRight,
            child1Margin: EdgeInsets.only(right: 64),
          ),
        ),
      ],
    );
  }

  Widget _buildAirportPart() {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: <Widget>[
        Text(
            '${safeString(widget.data.depAirport)}${safeString(widget.data.depTerm)}',
            style: TextStyle(
              color: Color(0xff666666),
              fontSize: 13,
            )),
        Text(
            '${safeString(widget.data.arrAirport)}${safeString(widget.data.arrTerm)}',
            style: TextStyle(
              color: Color(0xff666666),
              fontSize: 13,
            )),
      ],
    );
  }

  Widget _buildCityPart() {
    var cityPartChildren = <Widget>[];
    var cityLeft = Align(
      alignment: Alignment.centerLeft,
      child: FSuper(
        width: 106,
        text: widget.data.depCity,
        textColor: Color(0xff333333),
        textSize: 15,
        textAlignment: Alignment.centerLeft,
      ),
    );
    cityPartChildren.add(cityLeft);

    if (widget.showDate && widget.data.depTime != null) {
      DateTime date = DateTime.parse(widget.data.depTime);
      var cityDepDate = Text(
        '${date.month}月${date.day}日 ${weekdayCN(date.weekday)}',
        style: TextStyle(
          color: Color(0xff999999),
          fontSize: 13,
        ),
      );
      cityPartChildren.add(cityDepDate);
    }

    var cityRight = Align(
      alignment: Alignment.centerRight,
      child: FSuper(
        width: 106,
        text: widget.data.arrCity,
        textColor: Color(0xff333333),
        textSize: 15,
        textAlignment: Alignment.centerRight,
      ),
    );
    cityPartChildren.add(cityRight);

    return Stack(
      alignment: Alignment.center,
      children: cityPartChildren,
    );
  }

  Widget _buildAttentionPart() {
    return Align(
      alignment: Alignment.centerRight,
      child: Padding(
        padding: const EdgeInsets.only(top: 12, bottom: 12),
        child: Row(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            VerticalDivider(color: Color(0xffe0e0e0), width: 0.5),
            FSuper(
              width: 64.5,
              height: double.infinity,
              textAlignment: Alignment.center,
              text: widget.data.attention ? '已关注' : '关注',
              textColor:
                  widget.data.attention ? Color(0xffcccccc) : Color(0xfffca500),
              onClick: () {
                if (widget.attentionClicked != null) {
                  widget.attentionClicked();
                }
              },
            ),
          ],
        ),
      ),
    );
  }
}
