import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

const Color FLIGGY_COLOR = Color(0xFFFFDD00);
const Color DEFAULT_TEXT_COLOR = Color(0xFF5C5F66);
const Color BORDER_COLOR = Color(0xFF919499);

/*
  默认高度
 */
const double DEFAULT_IMAGE_HEIGHT = 160.0;
/*
  默认宽度
 */
const double DEFAULT_IMAGE_WIDTH = 280.0;

/*
  主错误文案字体默认大小
 */
const double MAIN_ERROR_DEFAULT_TEXT_SIZE = 16.0;

/*
  副错误文案字体默认大小
 */
const double VICE_ERROR_DEFAULT_TEXT_SIZE = 13.0;

/*
  所有按钮默认字体大小
 */
const double BTN_DEFAULT_TEXT_SIZE = 13.0;

/*
  去首页逛逛按钮宽度
 */
const double NO_GOODS_BTN_WIDTH = 100.0;

/*
  所有按钮默认高度
 */
const double BTN_DEFAULT_HEIGHT = 27.0;

/*
  按钮默认宽度
 */
const double BTN_DEFAULT_WIDTH = 83.0;

/*
  错误文案默认的padding top
 */
const double ERROR_TEXT_DEFAULT_PADDING_TOP = 8.0;

/*
  副错误文案默认的padding top
 */
const double VICE_ERROR_TEXT_DEFAULT_PADDING_TOP = 12.0;

/*
  按钮区域默认的padding top
 */
const double BTN_AREA_DEFAULT_PADDING_TOP = 18.0;

class FResult extends StatefulWidget {
  /*
   * 组件在被依赖项目中的相对路径
   */
  final String widgetPath;

  /*
   * 依赖该组件的项目的名称
   */
  final String packageName;

  /*
   * 错误文案
   */
  String errorText;

  /*
   * 是否显示错误文案
   */
  bool showErrorText;

  /*
   * 副错误文案
   */
  String viceErrorText;

  /*
   * 是否显示副错误按钮
   */
  bool showViceError;

  /*
   * 主按钮文案
   */
  String mainBtnText;

  /*
   * 是否显示按钮区域
   */
  bool showBtnContainer;

  /*
   * 副按钮文案
   */
  String viceBtnText;

  /*
   * 是否显示副按钮
   */
  bool showViceBtn;

  /*
   * 按钮回调
   */
  VoidCallback onButtonPress;

  /*
   * 副按钮回调
   */
  VoidCallback onViceBtnPress;

  /*
   * 默认图片路径
   */
  String _defaultImagePath;

  /*
   * 图片路径
   */
  String imagePath;

  /*
   * 背景色
   */
  Color btnColor;

  /*
    组件高度
   */
  final double imageHeight;

  /*
    组件宽度
   */
  final double imageWidth;

  /*
    主错误文案字体大小
  */
  final double errorTextSize;

  /*
    主错误文案字体大小
  */
  final double viceErrorTextSize;

  /*
    按钮字体大小
   */
  final double btnTextSize;

  /*
    去首页看看按钮宽度
   */
  final double singleBtnWidth;

  /*
    普通按钮宽度
   */
  final double normalBtnWidth;

  /*
    普通按钮高度
   */
  final double btnHeight;

  /*
    错误文案字体颜色
   */
  final Color errorTextColor;

  /*
    副错误文案字体颜色
   */
  final Color viceErrorTextColor;

  final double errorTextPaddingTop;

  final double viceErrorPaddingTop;

  final double btnPaddingTop;

  FResult.noGoods(String widgetPath, String packageName,
      {Key key,
      @required this.onButtonPress,
      this.imagePath,
      this.errorText,
      this.mainBtnText,
      this.viceErrorText,
      this.btnColor: FLIGGY_COLOR,
      this.imageHeight: DEFAULT_IMAGE_HEIGHT,
      this.imageWidth: DEFAULT_IMAGE_WIDTH,
      this.errorTextSize: MAIN_ERROR_DEFAULT_TEXT_SIZE,
      this.viceErrorTextSize: VICE_ERROR_DEFAULT_TEXT_SIZE,
      this.btnTextSize: BTN_DEFAULT_TEXT_SIZE,
      this.btnHeight: BTN_DEFAULT_HEIGHT,
      this.singleBtnWidth: NO_GOODS_BTN_WIDTH,
      this.errorTextColor: DEFAULT_TEXT_COLOR,
      this.viceErrorTextColor: BORDER_COLOR,
      this.errorTextPaddingTop: ERROR_TEXT_DEFAULT_PADDING_TOP,
      this.viceErrorPaddingTop: VICE_ERROR_TEXT_DEFAULT_PADDING_TOP,
      this.btnPaddingTop: BTN_AREA_DEFAULT_PADDING_TOP})
      : widgetPath = widgetPath,
        packageName = packageName,
        this.showErrorText = true,
        this.showBtnContainer = true,
        this.showViceError = true,
        this.showViceBtn = false,
        this.viceBtnText = "",
        this._defaultImagePath = "images/no_goods.png",
        this.normalBtnWidth = 0,
        super(key: key) {
    if (null == this.errorText) {
      this.errorText = "购物车空空如也";
    }
    if (null == this.mainBtnText) {
      this.mainBtnText = "刷新";
    }
    if (null == this.viceErrorText) {
      this.viceErrorText = "\"世界这么大，再忙也要到处去走走\"";
    }
  }

  FResult.error(String widgetPath, String packageName,
      {Key key,
      @required this.onButtonPress,
      @required this.onViceBtnPress,
      this.imagePath,
      this.btnColor,
      this.errorText,
      this.mainBtnText,
      this.viceErrorText,
      this.viceBtnText,
      this.imageHeight: DEFAULT_IMAGE_HEIGHT,
      this.imageWidth: DEFAULT_IMAGE_WIDTH,
      this.errorTextSize: MAIN_ERROR_DEFAULT_TEXT_SIZE,
      this.viceErrorTextSize: VICE_ERROR_DEFAULT_TEXT_SIZE,
      this.btnTextSize: BTN_DEFAULT_TEXT_SIZE,
      this.btnHeight: BTN_DEFAULT_HEIGHT,
      this.normalBtnWidth = BTN_DEFAULT_WIDTH,
      this.errorTextColor: DEFAULT_TEXT_COLOR,
      this.viceErrorTextColor: BORDER_COLOR,
      this.errorTextPaddingTop: ERROR_TEXT_DEFAULT_PADDING_TOP,
      this.viceErrorPaddingTop: VICE_ERROR_TEXT_DEFAULT_PADDING_TOP,
      this.btnPaddingTop: BTN_AREA_DEFAULT_PADDING_TOP})
      : widgetPath = widgetPath,
        packageName = packageName,
        this.showErrorText = true,
        this.showViceError = true,
        this.showBtnContainer = true,
        this.showViceBtn = true,
        this._defaultImagePath = "images/error.png",
        this.singleBtnWidth = 0,
        super(key: key) {
    if (null == this.errorText) {
      this.errorText = "系统竟然崩溃了";
    }
    if (null == this.viceErrorText) {
      this.viceErrorText = "\"别着急，试试看刷新页面~\"";
    }
    if (null == this.mainBtnText) {
      this.mainBtnText = "刷新";
    }
    if (null == this.viceBtnText) {
      this.viceBtnText = "报错";
    }
  }

  FResult.noNetwork(String widgetPath, String packageName,
      {Key key,
      @required this.onButtonPress,
      @required this.onViceBtnPress,
      this.imagePath,
      this.btnColor,
      this.errorText,
      this.mainBtnText,
      this.viceErrorText,
      this.imageHeight: DEFAULT_IMAGE_HEIGHT,
      this.imageWidth: DEFAULT_IMAGE_WIDTH,
      this.errorTextSize: MAIN_ERROR_DEFAULT_TEXT_SIZE,
      this.viceErrorTextSize: VICE_ERROR_DEFAULT_TEXT_SIZE,
      this.btnTextSize: BTN_DEFAULT_TEXT_SIZE,
      this.btnHeight: BTN_DEFAULT_HEIGHT,
      this.normalBtnWidth = BTN_DEFAULT_WIDTH,
      this.errorTextColor: DEFAULT_TEXT_COLOR,
      this.viceErrorTextColor: BORDER_COLOR,
      this.errorTextPaddingTop: ERROR_TEXT_DEFAULT_PADDING_TOP,
      this.viceErrorPaddingTop: VICE_ERROR_TEXT_DEFAULT_PADDING_TOP,
      this.btnPaddingTop: BTN_AREA_DEFAULT_PADDING_TOP})
      : widgetPath = widgetPath,
        packageName = packageName,
        this.showErrorText = true,
        this.showViceError = true,
        this.showBtnContainer = true,
        this.showViceBtn = false,
        this.viceBtnText = "",
        this._defaultImagePath = "images/no_network.png",
        this.singleBtnWidth = normalBtnWidth,
        super(key: key) {
    if (null == this.errorText) {
      this.errorText = "网络竟然崩溃了";
    }
    if (null == this.viceErrorText) {
      this.viceErrorText = "\"别紧张，试试换个姿势刷新页面\"";
    }
    if (null == this.mainBtnText) {
      this.mainBtnText = "刷新";
    }

  }

  FResult.custom(String widgetPath, String packageName,
      {Key key,
      @required this.onButtonPress,
      @required this.onViceBtnPress,
      @required this.errorText,
      this.showErrorText: true,
      @required this.viceErrorText,
      this.showViceError: true,
      @required this.mainBtnText,
      this.showBtnContainer: true,
      this.showViceBtn: true,
      this.viceBtnText,
      @required this.imagePath,
      this.btnColor,
      this.imageHeight: DEFAULT_IMAGE_HEIGHT,
      this.imageWidth: DEFAULT_IMAGE_WIDTH,
      this.errorTextSize: MAIN_ERROR_DEFAULT_TEXT_SIZE,
      this.viceErrorTextSize: VICE_ERROR_DEFAULT_TEXT_SIZE,
      this.btnTextSize: BTN_DEFAULT_TEXT_SIZE,
      this.btnHeight: BTN_DEFAULT_HEIGHT,
      this.normalBtnWidth = BTN_DEFAULT_WIDTH,
      this.singleBtnWidth: NO_GOODS_BTN_WIDTH,
      this.errorTextColor: DEFAULT_TEXT_COLOR,
      this.viceErrorTextColor: BORDER_COLOR,
      this.errorTextPaddingTop: ERROR_TEXT_DEFAULT_PADDING_TOP,
      this.viceErrorPaddingTop: VICE_ERROR_TEXT_DEFAULT_PADDING_TOP,
      this.btnPaddingTop: BTN_AREA_DEFAULT_PADDING_TOP})
      : widgetPath = widgetPath,
        packageName = packageName,
        assert(null != errorText),
        assert(null != imagePath),
        super(key: key) {

    if (null == errorText) {
      showErrorText = false;
    }
    if (null == viceErrorText) {
      showViceError = false;
    }
    if (null == viceBtnText) {
      showViceBtn = false;
    }
  }

  @override
  State<StatefulWidget> createState() => _FResult();
}

class _FResult extends State<FResult> {
  @override
  Widget build(BuildContext context) {
    bool useDefaultImage = widget.imagePath == null;

    return Center(
        child: Column(children: <Widget>[
      SizedBox(
          height: widget.imageHeight,
          width: widget.imageWidth,
          child: useDefaultImage
              ? Image.asset(
                  widget.widgetPath + widget._defaultImagePath,
                  package: widget.packageName,
                )
              : Image.asset(widget.widgetPath + widget.imagePath)),
      Padding(
        padding: EdgeInsets.only(top: widget.errorTextPaddingTop),
        child: Offstage(
            offstage: !widget.showErrorText,
            child: Text(widget.errorText,
                style: TextStyle(
                    color: widget.errorTextColor,
                    fontSize: widget.errorTextSize))),
      ),
      Padding(
          padding: EdgeInsets.only(top: widget.viceErrorPaddingTop),
          child: Offstage(
              offstage: !widget.showViceError,
              child: Text(widget.viceErrorText,
                  style: TextStyle(
                      color: widget.viceErrorTextColor,
                      fontSize: widget.viceErrorTextSize)))),
      Offstage(
          offstage: !widget.showBtnContainer,
          child: Padding(
              padding: EdgeInsets.only(top: widget.btnPaddingTop),
              child: widget.showViceBtn ? _twoBtnWidget() : _singleBtn()))
    ]));
  }

  Widget _singleBtn() {
    return SizedBox(
        height: widget.btnHeight,
        width: widget.singleBtnWidth,
        child: RaisedButton(
          color: widget.btnColor,
          highlightColor: widget.btnColor,
          colorBrightness: Brightness.light,
          splashColor: widget.btnColor,
          child: Text(widget.mainBtnText,
              style: TextStyle(
                  color: widget.errorTextColor, fontSize: widget.btnTextSize)),
          shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(20.0),
              side: BorderSide(
                color: BORDER_COLOR,
                width: 0.5,
                style: BorderStyle.solid,
              )),
          onPressed: _onBtnPressed,
        ));
  }

  _onBtnPressed() {
    if (widget.onButtonPress != null) {
      widget.onButtonPress();
    }
  }

  _onViceBtnPressed() {
    if (widget.onViceBtnPress != null) {
      widget.onViceBtnPress();
    }
  }

  Widget _twoBtnWidget() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.only(right: 6.0),
          child: SizedBox(
              height: widget.btnHeight,
              width: widget.normalBtnWidth,
              child: RaisedButton(
                color: widget.btnColor,
                child: Text(widget.mainBtnText,
                    style: TextStyle(
                        color: widget.errorTextColor,
                        fontSize: widget.btnTextSize)),
                shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(20.0),
                    side: BorderSide(
                      color: BORDER_COLOR,
                      width: 0.5,
                      style: BorderStyle.solid,
                    )),
                onPressed: _onBtnPressed,
              )),
        ),
        Padding(
          padding: const EdgeInsets.only(left: 6.0),
          child: SizedBox(
              height: widget.btnHeight,
              width: widget.normalBtnWidth,
              child: RaisedButton(
                color: widget.btnColor,
                child: Text(widget.viceBtnText,
                    style: TextStyle(
                        color: widget.errorTextColor,
                        fontSize: widget.btnTextSize)),
                shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(20.0),
                    side: BorderSide(
                      color: BORDER_COLOR,
                      width: 0.5,
                      style: BorderStyle.solid,
                    )),
                onPressed: _onViceBtnPressed,
              )),
        ),
      ],
    );
  }
}
