import 'dart:async';

import 'package:fflight/widget/triangle_clipper.dart';
import 'package:flutter/material.dart';

import 'fsuper/fsuper.dart';

class FTip {
  double width;
  double height;
  String text;
  TextStyle textStyle;
  EdgeInsets padding;
  double topOffset;
  double leftOffset;
  bool canClickDismiss;
  bool autoDismiss;
  Duration autoDismissDuration;
  bool needUpdate = false;
  double triangleOffset;
  Color color;

  FTip({
    this.width,
    this.height,
    @required this.text = '',
    this.textStyle,
    this.topOffset = 0.0,
    this.leftOffset = 0.0,
    this.canClickDismiss = true,
    this.autoDismiss = false,
    this.autoDismissDuration,
    this.triangleOffset,
    this.padding,
    this.color,
  });

  OverlayEntry _overlayEntry;

  FTip show(BuildContext context, {bool update = false}) {
    if (context == null) return this;
    if (_overlayEntry != null) {
      if (update) {
        _overlayEntry.remove();
        _overlayEntry = null;
      } else {
        return this;
      }
    }
    OverlayState overlayState = Overlay.of(context);
    _overlayEntry = OverlayEntry(
        builder: (BuildContext context) => Positioned(
              left: leftOffset,
              top: topOffset,
              child: FSuper(
                width: width,
                height: height,
                textAlign: TextAlign.start,
                spans: [
                  TextSpan(
                    text: text ?? '',
                    style: textStyle ??
                        TextStyle(
                            fontSize: 13,
                            color: Colors.white,
                            decoration: TextDecoration.none),
                  )
                ],
                padding: padding ?? EdgeInsets.fromLTRB(15, 9, 15, 9),
                backgroundColor: color ?? Color(0x99000000),
                corner: Corner.all(5),
                onClick: canClickDismiss
                    ? () {
                        _overlayEntry.remove();
                        _overlayEntry = null;
                      }
                    : null,
                child1: ClipPath(
                  clipper: TriangleClipper(),
                  child: Container(
                    width: 12,
                    height: 6,
                    color: color ?? Color(0x99000000),
                  ),
                ),
                child1Alignment: triangleOffset == null
                    ? Alignment.topCenter
                    : Alignment.topLeft,
                child1Margin:
                    EdgeInsets.only(left: triangleOffset ?? 0, top: -5.9),
              ),
            ));
    overlayState.insert(_overlayEntry);
    if (autoDismiss) {
      Timer(autoDismissDuration ?? Duration(milliseconds: 2000), () {
        if (_overlayEntry != null) {
          _overlayEntry.remove();
          _overlayEntry = null;
        }
      });
    }
    return this;
  }

  void dispose() {
    if (_overlayEntry != null) {
      _overlayEntry.remove();
      _overlayEntry = null;
    }
  }
}
